local PLAYER_UNITS = {
	player = true,
	vehicle = true,
	pet = true,
};

function ArenaLive_AuraFrame_Initialise(self, addonName, frameType, buffFrame, debuffFrame, buffTemplate, debuffTemplate)
	-- Create a shortcut for the parent frame.
	local parent = self:GetParent();
	local parentName = parent:GetName();
	parent.auraFrame = self;
	
	self.addonName = addonName;
	self.frameType = frameType;
	self.buffFrame = buffFrame;
	self.debuffFrame = debuffFrame;
	self.buffTemplate = buffTemplate;
	self.debuffTemplate = debuffTemplate;

	self:RegisterEvent("UNIT_AURA");
end



function ArenaLive_AuraFrame_UpdateUnit (self, unit)

	self.unit = unit;
	self.buffFrame.unit = unit;
	self.debuffFrame.unit = unit;
	self:SetScript("OnEvent", ArenaLive_AuraFrame_OnEvent);
	self:Show();
	
	ArenaLive_AuraFrame_UpdateAuras(self);

end



function ArenaLive_AuraFrame_ResetFrame (self)

	self.unit = nil;
	self.buffFrame.unit = nil;
	self.debuffFrame.unit = nil;
	self:SetScript("OnEvent", nil);
	self:Hide();
end



function ArenaLive_AuraFrame_OnEvent(self, event, ...)
	
	local unit = ...;

	if (not self.unit or unit ~= self.unit or not self:IsVisible() ) then
		return;
	end
	
	ArenaLive_AuraFrame_UpdateAuras(self)

end

-- Outside the function to reduce garbage
local largeBuffList = {};
local largeDebuffList = {};
local color = {};
function ArenaLive_AuraFrame_UpdateAuras (self)
	
	if ( not self.unit ) then
		return;
	end
	
	local prefix = self.buffFrame:GetName();
	local maxRowWidth = self:GetSize();
	local frame, frameName;
	local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, _, spellID;
	local numBuffs = 0;
	local maxBuffs = 40; -- For tournamentstyle mode
	
	local maxShownBuffs = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["maxShownBuffs"];
	local maxShownDebuffs = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["maxShownDebuffs"];
	local onlyShowCastableBuffs = ARENALIVE_DATABASE[self.addonName]["options"]["onlyShowCastableBuffs"];
	local onlyShowDispellableDebuffs = ARENALIVE_DATABASE[self.addonName]["options"]["onlyShowDispellableDebuffs"];
	local showAllEnemyDebuffs = ARENALIVE_DATABASE[self.addonName]["options"]["showAllEnemyDebuffs"];
	local filterBuffsTournamentStyle = ARENALIVE_DATABASE[self.addonName]["options"]["filterBuffsTournamentStyle"]; -- DO NOT USE THIS FOR NORMAL UNITFRAMES
	
	local playerIsUnit = UnitIsUnit("player", self.unit);
	local canAssist = UnitCanAssist("player", self.unit);

	local filter;
	if ( onlyShowCastableBuffs and canAssist ) then
		filter = "RAID";
	end	
	
	-- Buff handler
	local frameID = 1;
	local auraID = 1;
	while frameID <= maxShownBuffs do
		
		frameName = prefix.."Buff"..frameID;
		frame = _G[frameName];

		
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, _, spellID = UnitBuff(self.unit, auraID, filter);
		if ( name ) then
			if ( ArenaLive_Aura_ShouldShowBuff(self.unit, auraID, filterBuffsTournamentStyle, filter) ) then
				if ( not frame ) then
					if ( icon ) then
						frame = ArenaLive_Aura_CreateAuraFrame(self, frameName, frameID, "BUFF");
					end
				end
				
				if ( icon ) then
					frame.unit = self.unit;
					frame:SetID(auraID);
					
					frame.icon:SetTexture(icon);
					
					if ( count > 1 ) then
						frame.count:SetText(count);
						frame.count:Show();
					else
						frame.count:Hide();
					end
					
					if ( duration > 0 ) then
						frame.cooldown:Show();
						CooldownFrame_SetTimer(frame.cooldown, expires - duration, duration, 1);
					else
						frame.cooldown:Hide();
					end
					
					if ( not playerIsUnit and isStealable and not filterBuffsTournamentStyle ) then
						frame.stealable:Show();
					else
						frame.stealable:Hide();
					end

					-- set the buff to be big if the buff is cast by the player or his pet
					largeBuffList[auraID] = PLAYER_UNITS[caster];				
					
					numBuffs = numBuffs + 1;
					frameID = frameID + 1;
					
					frame:ClearAllPoints();
					frame:Show();				
				end
			end
		end
		
		-- Hide alle unnecessary Frames
		if ( not icon ) then
			if ( frame ) then
				frame:Hide();
				frameID = frameID + 1;
			else
				break;
			end
		end
		
		auraID = auraID + 1;
		
	end

	if ( onlyShowDispellableDebuffs and canAssist ) then
		filter = "RAID";
	else
		filter = nil;
	end		
	
	-- Debuffhandler
	local numDebuffs = 0;
	frameID = 1;
	auraID = 1;
	local prefix = self.debuffFrame:GetName();
	
	while frameID <= maxShownDebuffs do
		
		frameName = prefix.."Debuff"..frameID;
		frame = _G[frameName];
		
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, _, spellID = UnitDebuff(self.unit, auraID, filter);
		if ( name ) then
			if ( ArenaLive_Aura_ShouldShowDebuff (self.unit, auraID, showAllEnemyDebuffs, filter) ) then
				if ( not frame ) then
					if ( icon ) then
						frame = ArenaLive_Aura_CreateAuraFrame(self, frameName, frameID, "DEBUFF");
					end
				end
				
				if ( icon ) then
					frame.unit = self.unit;
					
					frame:SetID(auraID);
					
					frame.icon:SetTexture(icon);
					
					if ( count > 1 ) then
						frame.count:SetText(count);
						frame.count:Show();
					else
						frame.count:Hide();
					end
					
					if ( duration > 0 ) then
						frame.cooldown:Show();
						CooldownFrame_SetTimer(frame.cooldown, expires - duration, duration, 1);
					else
						frame.cooldown:Hide();
					end
					
					if ( dispelType ) then
						color = DebuffTypeColor[dispelType];
					else
						color = DebuffTypeColor["none"];
					end
					
					frame.border:SetVertexColor(color.r, color.g, color.b);
					
					-- set the debuff to be big if the buff is cast by the player or his pet
					largeDebuffList[auraID] = (PLAYER_UNITS[caster]);
					
					numDebuffs = numDebuffs + 1;
					frameID = frameID + 1;
					
					frame:ClearAllPoints();
					frame:Show();			
				end			
			end
		end
		
		-- Hide alle unnecessary Frames
		if ( not icon ) then
			if ( frame ) then
				frame:Hide();
				frameID = frameID + 1;
			else
				break;
			end
		end
		
		auraID = auraID + 1;		
	end
	local buffFrameheight, debuffFrameheight, newAuraFrameHeight;
	-- Update Buff Positions, we get the height of the frame as a return value.
	buffFrameheight = ArenaLive_Aura_UpdateAuraPositions(self, self.buffFrame, numBuffs, largeBuffList, maxRowWidth);
	-- Update Debuff Positions, we get the height of the frame as a return value.
	debuffFrameheight = ArenaLive_Aura_UpdateAuraPositions(self, self.debuffFrame, numDebuffs, largeDebuffList, maxRowWidth);
	
	ArenaLive_Aura_AuraFrameUpdate(self, maxRowWidth, numBuffs, numDebuffs, buffFrameheight, debuffFrameheight);
	
end



function ArenaLive_Aura_CreateAuraFrame(self, frameName, index, auraType)

	local frame;
	
	if ( auraType == "DEBUFF" ) then -- Debuff
		ArenaLive_SystemMessage("Create new Debuff Frame icon: "..self:GetName()..", #"..index, true);
		frame = CreateFrame("Button", frameName, self.debuffFrame, self.debuffTemplate)
		self.debuffFrame["aura"..index] = frame;
		frame.icon = _G[frameName.."Icon"];
		frame.count = _G[frameName.."Count"];
		frame.cooldown = _G[frameName.."Cooldown"];
		frame.border = _G[frameName.."Border"];	
	
	elseif ( auraType == "BUFF" ) then -- Buff
		ArenaLive_SystemMessage("Create new Buff Frame icon: "..self:GetName()..", #"..index, true);
		frame = CreateFrame("Button", frameName, self.buffFrame, self.buffTemplate)
		self.buffFrame["aura"..index] = frame;
		frame.icon = _G[frameName.."Icon"];
		frame.count = _G[frameName.."Count"];
		frame.cooldown = _G[frameName.."Cooldown"];
		frame.stealable = _G[frameName.."Stealable"];
	end
	
	return frame;

end



function ArenaLive_Aura_ShouldShowBuff (unit, auraID, isTournamentStyle, filter)

	if ( isTournamentStyle ) then
		
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, isCastByPlayer = UnitBuff(unit, auraID, filter);
		
		if ( ARENALIVE_SPELLS["ShownBuffs"][spellId] ) then
			return 
		else
			return nil;
		end
		
	else
		return true;
	end

end



function ArenaLive_Aura_ShouldShowDebuff (unit, index, showAllEnemyDebuffs, filter)

	if ( (showAllEnemyDebuffs) or ( not UnitCanAttack("player", unit) ) ) then
		return true;
	else
		
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, isCastByPlayer = UnitDebuff(unit, index, filter);
		
		local hasCustom, alwaysShowMine, showForMySpec = SpellGetVisibilityInfo(spellID, "ENEMY_TARGET");
		
		if ( hasCustom ) then
			return showForMySpec or (alwaysShowMine and (unitCaster == "player" or unitCaster == "pet" or unitCaster == "vehicle") );
		else
			return not isCastByPlayer or unitCaster == "player" or unitCaster == "pet" or unitCaster == "vehicle";
		end
	end

end



function ArenaLive_Aura_UpdateAuraPositions (self, auraFrame, numAuras, largeAuraList, maxRowWidth)

	local iconSpacing = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["iconSpacing"];
	local iconSize = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["iconSize"];
	local largeIconSize = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["largeIconSize"];
	local frame, relativeFrame, size;
	local rowWidth = 0;
	local newRowYOffset = 0;
	local totalFrameHeight = 0;
	local rowLargestIconSize = 0;
	
	for i = 1, numAuras do
	
		frame = auraFrame["aura"..i];
		
		if ( largeAuraList[i] ) then
		
			size = largeIconSize;
		else
			size = iconSize;
		end
		
		if ( i == 1 ) then
			rowWidth = size;
			rowLargestIconSize = size;
		else
			rowWidth = rowWidth + size + iconSpacing;
		end

		if ( rowWidth > maxRowWidth ) then
			-- We begin a new row here. We set the totalFrameHeight to the frameheight + the largest Icon of the preceding row to get the yOffset we need.
			newRowYOffset = newRowYOffset + iconSpacing + rowLargestIconSize;
			rowWidth = size;
			rowLargestIconSize = 0;
			
			ArenaLive_Aura_UpdateAuraAnchor(self, auraFrame, i, i, size, iconSpacing, newRowYOffset);
			
			-- if the last icon creates a new row, we add his size to the newRowYOffset to get the overall height of the auraFrame.
			if ( i == numAuras ) then
				totalFrameHeight = newRowYOffset + size;
			end
		else
			
			ArenaLive_Aura_UpdateAuraAnchor(self, auraFrame, i, i-1, size, iconSpacing, newRowYOffset);
			
			-- The last Icon creates no new row, so we add the largestIconSize to newRowYOffset to get the total height.
			if ( i == numAuras ) then
				totalFrameHeight = newRowYOffset + rowLargestIconSize;
			end
		end
		
		-- Check if the icon is larger than the current largest icon.
		if ( rowLargestIconSize < size ) then
			rowLargestIconSize = size;
		end
		
	end
	
	return totalFrameHeight;
end

function ArenaLive_Aura_UpdateAuraAnchor(self, auraFrame, index, relativeIndex, size, xOffset, yOffset)

	local point, relativeTo, relativePoint;
	local growUpwards = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["growUpwards"];
	local rtlAuras = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["rtlAuras"];
	local frame = auraFrame["aura"..index];
	
	if ( growUpwards ) then
		point = "BOTTOM";
		relativePoint = "BOTTOM";
	else
		point = "TOP";
		relativePoint = "TOP"
		
		-- Because we grow downwards, yOffset must be a negative value.
		yOffset = -yOffset;
	end
	
	if ( rtlAuras ) then
		point = point.."RIGHT";
		relativePoint = relativePoint.."LEFT";
		xOffset = -xOffset
	else
		point = point.."LEFT";
		relativePoint = relativePoint.."RIGHT"
	end
	
	if ( index == 1 ) then
	
		relativePoint = point;
		xOffset = 0;
		yOffset = 0;
		relativeTo = frame:GetParent();
		
	elseif ( relativeIndex ~= index - 1 ) then
		--[[ The relativeIndex isn't the icon before, this means that we've started a new row. So we reset the relativePoint, because if relativePoint is nil, :SetPoint will set the same value as point automatically
			We also reset the xOffset, since we're at the beginning of a row.]]--
		relativePoint = point;
		relativeTo = frame:GetParent();
		xOffset = 0;
	else
		-- We set our relative Frame (the preceding buff) and reset yOffset, because positioning is relative to the preceding frame. 
		relativeTo = auraFrame["aura"..relativeIndex];
		yOffset = 0;
	end
	
	-- Set up the frame. The Aura is finally done!
	frame:SetSize(size, size);
	if (frame.stealable) then
		frame.stealable:SetSize(size+5, size+5);
	end
	frame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset);
end

function ArenaLive_Aura_AuraFrameUpdate(self, width, numBuffs, numDebuffs, buffFrameheight, debuffFrameheight)
	
	local point, relativeTo, relativePoint, totalHeight;
	local growUpwards = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["growUpwards"];
	local iconSpacing = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["auraFrame"]["iconSpacing"];
	local filterBuffsTournamentStyle = ARENALIVE_DATABASE[self.addonName]["options"]["filterBuffsTournamentStyle"]; -- DO NOT USE THIS FOR NORMAL UNITFRAMES
	local point;
	local yOffset;

	
	if ( growUpwards ) then
		point = "BOTTOMLEFT";
		relativePoint = "TOPLEFT"
		yOffset = iconSpacing;
	else
		point = "TOPLEFT";
		relativePoint = "BOTTOMLEFT";
		yOffset = -iconSpacing;
	end	
	
	if ( numBuffs == 0 ) then
		buffFrameheight = 1;
		yOffset = 0;
	end	
	
	if ( numDebuffs == 0 ) then
		debuffFrameheight = 1;
		yOffset = 0;
	end
	
	self.buffFrame:ClearAllPoints();
	self.debuffFrame:ClearAllPoints();
	
	-- Unit is friendly, so buffs come first. We also order buffs first if we are in a Spectator/Tournament environment.
	if ( ( UnitIsFriend("player", self.unit) ) or ( filterBuffsTournamentStyle ) ) then	
		self.buffFrame:SetPoint(point, self, point, 0, 0);
		self.debuffFrame:SetPoint(point, self.buffFrame, relativePoint, 0, yOffset);

	else
		
		self.debuffFrame:SetPoint(point, self, point, 0, 0);
		self.buffFrame:SetPoint(point, self.debuffFrame, relativePoint, 0, yOffset);	
	
	end
	
	totalHeight = buffFrameheight + debuffFrameheight + iconSpacing;
	
	if ( totalHeight == 0 ) then
		totalHeight = 1;
	end
	
	self.buffFrame:SetSize(width, buffFrameheight);
	self.debuffFrame:SetSize(width, debuffFrameheight);
	self:SetSize(width, totalHeight);

end