local _, L = ...;

function ArenaLive_CastBar_Initialise (self, addonName, frameType, border, borderShield, barIcon, text, animationGroup, fadeOutAnimation, showTradeSkills, showShield)
	-- Create a shortcut for the parent frame.
	local parent = self:GetParent();
	parent.castBar = self;
	
	self.addonName = addonName;
	self.frameType = frameType;
	self.border = border;
	self.borderShield = borderShield;
	self.icon = barIcon;
	self.text = text;
	self.animationGroup = animationGroup;
	self.fadeOutAnimation = fadeOutAnimation;
	self.showTradeSkills = showTradeSkills;
	self.showShield = showShield;
	self.casting = nil;
	self.channeling = nil;
	self.value = 0;
	self.maxValue = 0;
	self.minValue = 0;
	
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
	self:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
end



function ArenaLive_CastBar_UpdateUnit (self, unit)

	self.unit = unit;
	self:SetScript("OnEvent", ArenaLive_CastBar_OnEvent);
	self:SetScript("OnUpdate", ArenaLive_CastBar_OnUpdate);

end



function ArenaLive_CastBar_ResetFrame(self)

	self.unit = nil;
	self:SetScript("OnEvent", nil);
	self:SetScript("OnUpdate", nil);	
	self:Hide();
end



function ArenaLive_CastBar_OnShow (self)

	if ( not self.unit ) then
		return;
	end

	if ( self.casting ) then
		local _, _, _, _, startTime = UnitCastingInfo(self.unit);
		if ( startTime ) then
			self.value = (GetTime() - (startTime / 1000));
		end
	else
		local _, _, _, _, _, endTime = UnitChannelInfo(self.unit);
		if ( endTime ) then
			self.value = ((endTime / 1000) - GetTime());
		end
	end
end



function ArenaLive_CastBar_OnEvent(self, event, ... )
	
	if ( not self.unit ) then
		return;
	end

	local arg1 = ...;
	
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		local nameChannel  = UnitChannelInfo(self.unit);
		local nameSpell  = UnitCastingInfo(self.unit);
		
		if ( nameChannel ) then
			event = "UNIT_SPELLCAST_CHANNEL_START";
			arg1 = self.unit;
		elseif ( nameSpell ) then
			event = "UNIT_SPELLCAST_START";
			arg1 = self.unit;
		else
			self:Hide();
			ArenaLive_CastBar_FinishSpell(self, false);
		end
	end
	
	if ( arg1 ~= self.unit and event ~= "COMBAT_LOG_EVENT_UNFILTERED" ) then
		return;
	end
	
	if ( event == "UNIT_SPELLCAST_START" ) then
		ArenaLive_CastBar_StartCast(self, arg1, ...);
	elseif ( event == "UNIT_SPELLCAST_FAILED" ) or ( event == "UNIT_SPELLCAST_INTERRUPTED" ) then
		ArenaLive_CastBar_FailedOrInterruptedCast(self, event, ...);
	elseif ( event == "UNIT_SPELLCAST_DELAYED" ) then
		ArenaLive_CastBar_DelayedCast(self, ...);
	elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		ArenaLive_CastBar_SuccessfulSpell(self, ...);
	elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" ) then
		ArenaLive_CastBar_StartChannel(self, ...);
	elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" ) then
		ArenaLive_CastBar_UpdateChannel(self, ...);
	elseif ( event == "UNIT_SPELLCAST_CHANNEL_STOP" or event == "UNIT_SPELLCAST_STOP" ) then
		ArenaLive_CastBar_StopSpellOrChannel(self, event, ...);
	elseif ( self.showShield and event == "UNIT_SPELLCAST_INTERRUPTIBLE" ) then
		 ArenaLive_CastBar_ShowShieldBorder(self);
	elseif ( self.showShield and event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" ) then
		ArenaLive_CastBar_HideShieldBorder(self);
	elseif ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
		ArenaLive_CastBar_LockOutSpell(self, ...);
	end
	
end



function ArenaLive_CastBar_StartCast (self, unit, ...)

	if ( not unit ) then
		return;
	end
	
	local spellID = select(5, ...);
	local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo(self.unit);
	
	if (not name or (not self.showTradeSkill and isTradeSkill ) ) then
		self:Hide();
		return;
	end
	
	self:SetStatusBarColor(1.0, 0.7, 0.0);
	
	self.value = (GetTime() - (startTime / 1000));
	self.maxValue = (endTime - startTime) / 1000;
	self:SetMinMaxValues(0, self.maxValue);
	self:SetValue(self.value);	
	
	if ( self.text ) then
		self.text:SetText(text);
	end
	
	if ( self.icon ) then
		self.icon:SetTexture(texture);
	end	
	
	self:SetAlpha(1.0);
	self.casting = 1;
	if ( spellID ) then
		self.spellID = spellID;
	else
		self.spellName = name;
	end
	self.castID = castID;
	self.channeling = nil;

	if ( self.animationGroup:IsPlaying() ) then
		self.animationGroup:Stop();
	end
	
	if ( self.borderShield ) then
		if ( self.showShield and notInterruptible ) then
			self.borderShield:Show();
		else
			self.borderShield:Hide();
		end
	end
	
	if ( ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["castBar"]["isShown"] ) then
		self:Show();
	end

end



function  ArenaLive_CastBar_DelayedCast (self, ...)

	if ( self:IsShown() ) then
	
		local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitCastingInfo(self.unit);

		if ( not name or (not self.showTradeSkills and isTradeSkill)) then
				self:Hide();
				return;
		end
		
		self.value = (GetTime() - (startTime / 1000));
		self.maxValue = (endTime - startTime) / 1000;
		self:SetMinMaxValues(0, self.maxValue);	

		if ( not self.casting ) then
			self:SetStatusBarColor(1.0, 0.7, 0.0);

			self.casting = 1;
			self.channeling = nil;
		end
		
	end

end



function ArenaLive_CastBar_LockOutSpell(self, ...)

	local event = select(2, ...)

	if ( event == "SPELL_INTERRUPT" ) then

		local destName = select(9, ...);
		local spellSchool = select(17, ...);
		local unitName, unitRealm = UnitName(self.unit);
	
		if (unitRealm and unitRealm ~= "" ) then
			unitName = unitName.."-"..unitRealm;
		end

		if ( destName == unitName ) then
			self.text:SetText(string.format(L["LOCKOUT"], GetSchoolString(spellSchool)));
			ArenaLive_CastBar_FinishSpell(self, false, false)
		end
	end
	
end



function ArenaLive_CastBar_FailedOrInterruptedCast (self, event, ...)

	if ( self:IsShown() and (( self.casting and select(4, ...) == self.castID ) or (self.channeling))) then
	
		if ( self.text ) then
			if ( event == "UNIT_SPELLCAST_FAILED" ) then
				self.text:SetText(FAILED);
				self:SetStatusBarColor(1.0, 0.0, 0.0);
			else
				self.text:SetText(INTERRUPTED);
				self:SetStatusBarColor(1.0, 0.0, 0.0);
			end
		end		
		
		ArenaLive_CastBar_FinishSpell(self, false, false);	
	end

end



function ArenaLive_CastBar_SuccessfulSpell (self, ...)

	if ( self:IsShown() and ( self.casting and select(4, ...) == self.castID ) and not self.animationGroup:IsPlaying() ) then
		ArenaLive_CastBar_FinishSpell (self, false, true);
	end
end



function ArenaLive_CastBar_StartChannel (self, ...)

	local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(self.unit);

	if ( not name or (not self.showTradeSkills and isTradeSkill)) then
		self:Hide();
		return;
	end	

	self:SetStatusBarColor(0.0, 1.0, 0.0);
	self.value = ((endTime / 1000) - GetTime());
	self.maxValue = (endTime - startTime) / 1000;
	self:SetMinMaxValues(0, self.maxValue);
	self:SetValue(self.value);
	
	if ( self.text ) then
		self.text:SetText(name);
	end	

	if ( self.icon ) then
		self.icon:SetTexture(texture);
	end
	
	self:SetAlpha(1.0);
	self.casting = nil;
	self.channeling = 1;
	
	if ( self.animationGroup:IsPlaying() ) then
		self.animationGroup:Stop();
	end	
	
	if ( self.borderShield ) then
		if ( self.showShield and notInterruptible ) then
			self.borderShield:Show();
		else
			self.borderShield:Hide();
		end
	end
	
	if ( ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["castBar"]["isShown"] ) then
		self:Show();
	end	
	
end



function ArenaLive_CastBar_UpdateChannel (self, ...)

	if ( self:IsShown() ) then
		local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitChannelInfo(self.unit);
		
		if ( not name or (not self.showTradeSkills and isTradeSkill)) then
			self:Hide();
			return;
		end
		
		self.value = ((endTime / 1000) - GetTime());
		self.maxValue = (endTime - startTime) / 1000;
		self:SetMinMaxValues(0, self.maxValue);
		self:SetValue(self.value);
	end
end



function ArenaLive_CastBar_StopSpellOrChannel (self, event, ...)

	if ( not self:IsVisible() ) then
		self:Hide();
	end

	if ( ( self.casting and event == "UNIT_SPELLCAST_STOP" and select(4, ...) == self.castID ) or ( self.channeling and event == "UNIT_SPELLCAST_CHANNEL_STOP" ) ) then

		if ( event == "UNIT_SPELLCAST_CHANNEL_STOP" ) then
			ArenaLive_CastBar_FinishSpell(self, true, false)
		else
			ArenaLive_CastBar_FinishSpell(self, false, true)
		end
	
	end


end



function ArenaLive_CastBar_ShowShieldBorder(self)
	
	if ( self.borderShield ) then
		self.borderShield:Hide();
		if ( self.borderShield ) then
			self.borderShield:Show();
		end
	end
	
end



function ArenaLive_CastBar_HideShieldBorder(self)

	if ( self.borderShield ) then
		self.borderShield:Show();
		if ( self.borderShield ) then
			self.borderShield:Hide();
		end
	end

end



function ArenaLive_CastBar_FinishSpell (self, wasChannel, wasSuccessful)

	if ( wasSuccessful == true ) then
		self.casting = nil;
		self:SetStatusBarColor(0.0, 1.0, 0.0);
		self.fadeOutAnimation:SetStartDelay(0);
	elseif ( wasChannel == true ) then
		self.channeling = nil;
		self.fadeOutAnimation:SetStartDelay(0);
	else
		self.casting = nil;
		self:SetStatusBarColor(1.0, 0.0, 0.0);
		self.fadeOutAnimation:SetStartDelay(0.5);
	end
	self:SetValue(self.maxValue);
	self.animationGroup:Play();
end



function ArenaLive_CastBar_OnUpdate (self, elapsed)

	if ( self.casting ) then
		self.value = self.value + elapsed;
		
		if ( self.value >= self.maxValue ) then
			self:SetValue(self.maxValue);
			ArenaLive_CastBar_FinishSpell(self, false, true);
			return;
		end
		
		self:SetValue(self.value);
		
	elseif ( self.channeling ) then
		self.value = self.value - elapsed;
		if ( self.value <= 0 ) then
			ArenaLive_CastBar_FinishSpell (self, true, nil)
			return;
		end
		self:SetValue(self.value);
	end
end

-- Reverse Fill Handler
function ArenaLive_CastBar_SetBarReverseFill(castBar, state)

	if ( castBar ) then
		castBar:SetReverseFill(state);
	end
	
end