function ArenaLive_HealthBar_Initialise (healthBar, healthBarText, addonName, frameType)

	if ( not healthBar ) then
		return;
	end
	
	healthBar.frameType = frameType;
	healthBar.addonName = addonName;
	healthBar.showDeadorGhostText = true;
	ArenaLive_StatusBarText_SetStatusBarTextString(healthBar, healthBarText);
	
	healthBar:RegisterEvent("UNIT_MAXHEALTH");
end


function ArenaLive_HealthBar_SetBarColour (healthBar)

	if ( not healthBar or healthBar.lockColor ) then
		return;
	end

	local colourMode = ARENALIVE_DATABASE[healthBar.addonName]["frameTypes"][healthBar.frameType]["healthBarColourMode"];
	
	local isPlayer = UnitIsPlayer(healthBar.unit);
	-- If the Unit is an NPC we fall back to ReactionColour. If it still isn't working we use green.
	if ( colourMode == "byClass" and isPlayer ) then
	
		if ( not healthBar.unit) then
			healthBar:SetStatusBarColor(0, 1, 0);
		end
		
		local _, class = UnitClass(healthBar.unit);
		
		
		if ( class ) then		
			healthBar:SetStatusBarColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b);
		else
				healthBar:SetStatusBarColor(0, 1, 0);
		end
		
	elseif ( colourMode == "byReaction" ) then
		
		if ( not healthBar.unit) then
			healthBar:SetStatusBarColor(0, 1, 0);
		end	
		
		local red, green, blue = UnitSelectionColor(healthBar.unit);
		
		if ( red ) then
			healthBar:SetStatusBarColor(red, green, blue);
		else
			healthBar:SetStatusBarColor(0, 1, 0);
		end
	elseif ( colourMode == "smooth" ) then
		-- Luckily Blizzard has already written a perfect function for smooth health bar colour! We just adjust this to our needs.
		local r, g, b;
		local value = UnitHealth(healthBar.unit);
		local minValue = 0;
		local maxValue = UnitHealthMax(healthBar.unit);

		if ( (maxValue - minValue) > 0 ) then
			value = (value - minValue) / (maxValue - minValue);
		else
			value = 0;
		end
		
		if(value > 0.5) then
			r = (1.0 - value) * 2;
			g = 1.0;
		else
			r = 1.0;
			g = value * 2;
		end

		b = 0;
		
		healthBar:SetStatusBarColor(r, g, b);
		
	else
		healthBar:SetStatusBarColor(0, 1, 0);
	end
end



function ArenaLive_HealthBar_OnEvent (self, event, ...)
	
	if ( not self.ignoreNoUnit or UnitGUID(self.unit) ) then
		ArenaLive_HealthBar_Update(self, ...);
	end
	
end



function ArenaLive_HealthBar_OnUpdate (self, elapsed)
	if ( not self.disconnected and not self.lockValues) then
		local currValue = UnitHealth(self.unit);

		if ( currValue ~= self.currValue ) then
			if ( not self.ignoreNoUnit or UnitGUID(self.unit) ) then
				local colourMode = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["healthBarColourMode"];
				if ( colourMode == "smooth" ) then
					ArenaLive_HealthBar_SetBarColour(self);
				end
				self:SetValue(currValue);
				self.currValue = currValue;
				ArenaLive_AbsorbBar_Update(self:GetParent());
				ArenaLive_HealPredictionBar_Update(self:GetParent());
			end
		end
	end

end

function ArenaLive_HealthBar_OnValueChanged(self)
	local value = self:GetValue();
	local minValue, maxValue;
	
	if ( self.unit ) then
		maxValue = UnitHealthMax(self.unit);
		minValue = self:GetMinMaxValues();
	else
		minValue, maxValue = self:GetMinMaxValues();
	end
	ArenaLive_StatusBarText_UpdateText(self, ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["healthBarTextString"], minValue, maxValue, value);
	
end

function ArenaLive_HealthBar_Update (healthBar, unit)

	if ( not healthBar or healthBar.lockValues ) then
		return;
	end

	if ( unit == healthBar.unit ) then

		ArenaLive_HealthBar_SetBarColour(healthBar, healthBar.frameType)
	
		local maxValue = UnitHealthMax(unit);
		healthBar.forceHideText = false;
		
		if ( maxValue == 0 ) then
			maxValue = 1;
			healthBar.forceHideText = true;
		end
	
		healthBar:SetMinMaxValues(0, maxValue);

		healthBar.disconnected = not UnitIsConnected(unit);
		if ( healthBar.disconnected ) then
			if ( not healthBar.lockColor ) then
				healthBar:SetStatusBarColor(0.5, 0.5, 0.5);
			end
			healthBar:SetValue(maxValue);
			healthBar.currValue = maxValue;
		else
			local currValue = UnitHealth(unit);
			healthBar:SetValue(currValue);
			healthBar.currValue = currValue;
		end

	end
	
end



-- Function for dynamic unitID unit frames
function ArenaLive_HealthBar_Reset (healthBar)
	healthBar:SetMinMaxValues(0, 1);
	healthBar:SetValue(1);
	healthBar:SetStatusBarColor(0.5, 0.5, 0.5);
end