local addonName, L = ...;
function ArenaLive_Options_OnLoad(self)

	-- Create a panel in Blizzard's addon panel
	local version = ARENALIVE_DATABASE["options"]["version"];
	self.name = "ArenaLive";
	self.okay = ArenaLive_Options_Okay;
	self.cancel = function() end;
	InterfaceOptions_AddCategory(self);
	
	self:RegisterEvent("ADDON_LOADED");
end

function ArenaLive_Options_OnEvent(self, event, ...)
	local arg1 = ...;
	if ( event == "ADDON_LOADED" and arg1 == addonName ) then
		-- Option menu text:
		ArenaLive_OptionsPortraitOverlayTitle:SetText(L["portraitOverlayPriorities:"]);
		ArenaLive_OptionsPortraitOverlayDescription:SetText(L["portraitOverlayPrioritiesDescription"]);
		
		-- CheckButtons:
		ArenaLive_Options_CheckButtonInitialise(ArenaLive_OptionsDebug, "debug", "enableDebug");
		ArenaLive_Options_CheckButtonInitialise(ArenaLive_OptionsShowCooldownText, "showCooldownText", "showCooldownText");
		
		-- Editboxes:
		ArenaLive_Options_EditBoxInitialise(ArenaLive_OptionsSecCastHistoryIconDelay, "castHistoryIconDuration", "secCastHistoryIconDelay")
		
		--  Portrait Overlay Sliders
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayDefCDSlider, "defCD", "defensiveCooldowns:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayOffCDSlider, "burstCD", "offensiveCooldowns:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayStunSlider, "stun", "stuns:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlaySilenceSlider, "silence", "silence:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayCrowdControlSlider, "crowdControl", "crowdControl:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayRootSlider, "root", "roots:");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayDisarmSlider, "disarm", "disarms:");

		
	end
end



function ArenaLive_Options_Okay(self)
	
	-- CheckButtons:
	ARENALIVE_DATABASE["options"]["debug"] = ArenaLive_OptionsDebug:GetChecked();
	ARENALIVE_DATABASE["options"]["showCooldownText"] = ArenaLive_OptionsShowCooldownText:GetChecked();
	
	-- Editboxes:
	ARENALIVE_DATABASE["options"]["castHistoryIconDuration"] = tonumber(ArenaLive_OptionsSecCastHistoryIconDelay:GetText());
	
	--  Portrait Overlay Priorities
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["defCD"] = ArenaLive_OptionsPortraitOverlayDefCDSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["burstCD"] = ArenaLive_OptionsPortraitOverlayOffCDSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["stun"] = ArenaLive_OptionsPortraitOverlayStunSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["silence"] = ArenaLive_OptionsPortraitOverlaySilenceSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["crowdControl"] = ArenaLive_OptionsPortraitOverlayCrowdControlSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["root"] = ArenaLive_OptionsPortraitOverlayRootSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["disarm"] = ArenaLive_OptionsPortraitOverlayDisarmSlider:GetValue();
	
	ArenaLive_Database_UpdateSavedVariables(nil, ArenaLive_SavedVariables);
end



-- EDITBOXES
function ArenaLive_Options_EditBoxInitialise(self, boxType, localisationKey)

	local editBoxName = self:GetName();
	local textString = _G[editBoxName.."Text"];
	
	textString:SetText(L[localisationKey]);
	
	self:SetText(ARENALIVE_DATABASE["options"][boxType]);
	
	-- We need to reset the EditBox's cursor position, so that the text is shown correctly.
	self:SetCursorPosition(0);
	
end



-- CHECKBUTTONS
function ArenaLive_Options_CheckButtonInitialise(self, buttonType, localisationKey)
	local buttonName = self:GetName();
	local textString = _G[buttonName.."Text"];
	
	textString:SetText(L[localisationKey]);
	self:SetChecked(ARENALIVE_DATABASE["options"][buttonType]);
end



-- Slider
function ArenaLive_OptionsPortraitPrioSlider_Initialise(self, ccType, localisationKey)
	
	local prefix = self:GetName();
	local title = _G[prefix.."Title"];
	local currText = _G[prefix.."Curr"];
	local minText = _G[prefix.."Low"];
	local maxText = _G[prefix.."High"];
	local currValue = ARENALIVE_DATABASE["options"]["overlayPriorities"][ccType];
	local minValue, maxValue = self:GetMinMaxValues();
	
	title:SetText(L[localisationKey]);
	minText:SetText(minValue);
	maxText:SetText(maxValue);
	currText:SetText(currValue);
	self:SetValue(currValue);

end



function ArenaLive_OptionsPortraitPrioSlider_OnValueChanged(self)
	
	local prefix = self:GetName();
	local currText = _G[prefix.."Curr"];
	local currValue = self:GetValue();
	
	currText:SetText(currValue);
end