local addonName, L = ...;
local PopupDialogs = {};
local numPopups = 1;

PopupDialogs["ACTIONNEEDSUIRELOAD"] =
{
	title = addonName,
	text = L["actionNeedsUIReload"],
	button1 = L["okay"],
	button2 = L["cancel"],
	OnAccept = function(self) ReloadUI(); end,
	OnCancel = function(self) end,
	timeout = 0,
	whileDead = 1,
};

function ArenaLive_Popup_AddDialog(info)
end

function ArenaLive_Popup_OnClick(popup, buttonID)
	
	local info = PopupDialogs[popup.which];
	
	if (not info ) then
		return;
	end
	
	if ( buttonID == 1 ) then
		local onAccept = info.OnAccept;
		
		if ( onAccept ) then
			onAccept(popup);
		end
	elseif ( buttonID == 2 ) then
		local onCancel = info.OnCancel;
		
		if ( onCancel ) then
			onCancel(popup);
		end	
	end
	
	popup:Hide();

end

function ArenaLive_Popup_OnShow(self)

	if ( not self.which or not PopupDialogs[self.which] ) then
		return;
	end
	
	local info = PopupDialogs[self.which];
	-- Set up the Popup
	if (info.title ) then
		self.title:SetText(PopupDialogs[self.which]["title"]);
	else
		self.text:SetText("");
	end
	
	if ( info.text ) then
		self.text:SetText(PopupDialogs[self.which]["text"]);
	else
		self.text:SetText("");
	end
	
	if ( info.button1 ) then
		self.button1.text:SetText(PopupDialogs[self.which]["button1"]);
		self.button1:Show();
	else
		self.button1:Hide();
	end
	
	if ( info.button2 ) then
		self.button2.text:SetText(PopupDialogs[self.which]["button2"]);
		self.button2:Show();
	else
		self.button2:Hide();
	end
	
	-- Anchor the buttons according to the number of shown buttons.
	if ( self.button1:IsShown() and self.button2:IsShown() ) then
		self.button1:ClearAllPoints();
		self.button2:ClearAllPoints();
		
		self.button1:SetPoint("TOP", self.text, "BOTTOM", -65, -5);
		self.button2:SetPoint("LEFT", self.button1, "RIGHT", 5, 0);
	elseif ( self.button1:IsShown() and not self.button2:IsShown() ) then
		self.button1:ClearAllPoints("TOP", self.text, "BOTTOM", 0, -5);
	end
	
	-- Set Popup height according to all shown parts
	local textHeight = self.text:GetHeight();
	local buttonHeight = self.button1:GetHeight() +5; -- +5 for the anchor offset to the text.
	
	popupHeight = textHeight + buttonHeight + 32;
	
	self:SetHeight(popupHeight);
	
end

function ArenaLive_Popup_OnHide(self)

	-- Reset the Popup
	self.which = nil;
	self.text:SetText("");
end

local match, frame;
function ArenaLive_Popup_OpenPopup(which)

	if ( not PopupDialogs[which] ) then
		return;
	end
	
	local info = PopupDialogs[which];
	
	-- Get the first inactive popup
	
	for i = 1, numPopups do
		frame = _G["ArenaLive_Popup"..i];
		
		-- We already have a type of that popup, so we don't open a new popup of that type. Maybe change that that in the future.
		if ( frame:IsShown() and which == frame.which ) then
			return;
		end
		
		-- We've found an inactive Popup that we can use
		if ( frame and not frame:IsShown() ) then
			match = frame;
		end
	end
	
	-- If we didn't find a inactive popup, we need to create a new one
	if ( not match ) then
		match = ArenaLive_Popup_CreatePopup();
	end
	
	match.which = which;
	match:Show();
	match = nil;
	frame = nil;

end

local lastPopup, newPopup;
function ArenaLive_Popup_CreatePopup()
	
	local newID = numPopups + 1;
	local newPopupName = "ArenaLive_Popup"..newID;
	
	-- Create the frame.
	newPopup = CreateFrame("Frame", newPopupName, ArenaLiveFrame, "ArenaLive_PopupTemplate")
	lastPopup = _G["ArenaLive_Popup"..numPopups];
	-- Anchor it to the currently last popup
	newPopup:SetPoint("TOP", lastPopup, "BOTTOM", 0, -10)

	numPopups = numPopups + 1;
	return newPopup;
end