--[[ Note: Some of ArenaLive's UnitFrames need a dynamic unitID. 
Therefore I've divided the frame loading process into two different functions.
The first initialises the frame structure, while the second one fills
the frame with unitdata and registers the events. ]]--
-- SetUnitID

-- OnEnter
-- OnLeave
-- UpdateTooltip

-- MAIN UNITFRAME FUNCTIONS
function ArenaLive_UnitFrame_Initialise (self, addonName, frameType, background, border, name, portrait, threeDPortrait, portraitOverlay, healthBar, healthBarText, powerBar, powerBarText, threatIndicator, threatFeedbackUnit, healPredictionBar, absorbBar, absorbBarOverlay, absorbFullHPIndicator, targetIndicator)

	self.addonName = addonName;
	self.frameType = frameType;
	self.background = background;
	self.border = border;
	self.name = name;
	self.portrait = portrait;
	self.threeDPortrait = threeDPortrait;
	self.portraitOverlay = portraitOverlay;
	self.healthBar = healthBar;
	self.healthBarText = healthBarText;
	self.powerBar = powerBar;
	self.powerBarText = powerBarText;
	self.threatIndicator = threatIndicator;
	self.healPredictionBar = healPredictionBar;
	self.absorbBar = absorbBar;
	self.absorbBarOverlay = absorbBarOverlay;
	self.absorbFullHPIndicator = absorbFullHPIndicator;
	
	ArenaLive_HealthBar_Initialise(healthBar, healthBarText, addonName, frameType);
	ArenaLive_PowerBar_Initialise(powerBar, powerBarText, addonName, frameType);
	ArenaLive_AbsorbBar_Initialise(absorbBar, absorbBarOverlay, absorbFullHPIndicator);
	ArenaLive_ThreatIndicator_Initialise(self, threatIndicator, threatFeedbackUnit);
	ArenaLive_PortraitOverlay_Initialise(portraitOverlay, self, portrait);
	ArenaLive_TargetIndicator_Initialise(targetIndicator, self);
	
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UNIT_NAME_UPDATE");
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	self:RegisterEvent("UNIT_PORTRAIT_UPDATE");
	self:RegisterEvent("UNIT_MODEL_CHANGED");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("UNIT_FACTION");
	
	-- We use this to change Attributes, as soon as we are out of combat, if a self.unit change occurs infight.
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	
	if ( self.healPredictionBar ) then
		self:RegisterEvent("UNIT_MAXHEALTH");
		self:RegisterEvent("UNIT_HEAL_PREDICTION");
	end	
	
	if ( self.absorbBar ) then
		self:RegisterEvent("UNIT_ABSORB_AMOUNT_CHANGED");
	end
	
	-- If a fullscreen frame (e.g. like the WorldMapFrame) is shown, the PlayerModel resets. OnShow fixes this. 
	self:SetScript("OnShow", ArenaLive_UnitFrame_OnShow);

end



function ArenaLive_UnitFrame_UpdateUnit (self, unit)
	
	self.unit = unit;
	-- It is not allowed to change Attributes InFight, so we need to check
	if ( not InCombatLockdown() ) then
		self:SetAttribute("unit", unit);
		
		-- For if we want to show some frames even if the unitID isn't currently active
		if ( self.registerUnitWatch ) then
			RegisterUnitWatch(self);
		end
	else
		self.needAttributeUpdate = true;	
	end
	
	if ( self.portraitOverlay ) then
		self.portraitOverlay:SetScript("OnUpdate", ArenaLive_PortraitOverlay_OnUpdate);
	end
	
	if ( self.healthBar ) then
		self.healthBar.unit = unit;
		self.healthBar:SetScript("OnEvent", ArenaLive_HealthBar_OnEvent);
		self.healthBar:SetScript("OnUpdate", ArenaLive_HealthBar_OnUpdate);
	end

	if ( self.powerBar ) then
		self.powerBar.unit = unit;
		self.powerBar:SetScript("OnEvent", ArenaLive_PowerBar_OnEvent);
		self.powerBar:SetScript("OnUpdate", ArenaLive_PowerBar_OnUpdate);
	end

	ArenaLive_UnitFrame_Update(self);
end


-- Reset function for dynamic unitID unit frames
function ArenaLive_UnitFrame_ResetFrame (self)

	self.unit = nil;
	if ( not InCombatLockdown() ) then
		UnregisterUnitWatch(self);
		self:SetAttribute("unit",nil);
	else
		self.needAttributeUpdate = true;
	end
	
	
	if ( self.healthBar ) then
		self.healthBar.unit = nil;
		self.healthBar:SetScript("OnEvent", nil);
		self.healthBar:SetScript("OnUpdate", nil);
		ArenaLive_HealthBar_Reset(self.healthBar);
	end
	
	if ( self.healPredictionBar ) then
		ArenaLive_HealPredictionBar_Update(self)
	end
	
	if ( self.absorbBar ) then
		ArenaLive_AbsorbBar_Update(self);
	end
	
	if ( self.powerBar ) then
		self.powerBar.unit = nil;
		self.powerBar:SetScript("OnEvent", nil);
		self.powerBar:SetScript("OnUpdate", nil);
		ArenaLive_PowerBar_Reset(self.powerBar);
	end
	
	if ( self.threatIndicator ) then
		self.threatIndicator:Hide();
	end

end



function ArenaLive_UnitFrame_OnEvent (self, event, ...)
	
	if (not self.unit) then
		return;
	end

	if ( self.targetIndicator ) then
		ArenaLive_TargetIndicator_OnEvent(self, event, ...)
	end	
	
	local arg1 = select(1, ...)

	if ( event == "ADDON_LOADED" and arg1 == self.addonName ) then
		-- For some reason, the player unitframe is shown as offline, when the frame is loaded on the initial login. I don't know why this is the case, but this way the problem is fixed.
		ArenaLive_UnitFrame_Update(self);
	elseif ( event == "UNIT_NAME_UPDATE" ) then
		ArenaLive_UnitFrame_NameUpdate(self);
	elseif ( ( event == "UNIT_MODEL_CHANGED" or event == "UNIT_PORTRAIT_UPDATE" ) and arg1 == self.unit ) then
		ArenaLive_UnitFrame_PortraitUpdate(self);
	elseif ( event == "UNIT_MAXHEALTH" and arg1 == self.unit  ) then
		ArenaLive_HealPredictionBar_Update(self);
		ArenaLive_AbsorbBar_Update(self);
	elseif ( event == "UNIT_HEAL_PREDICTION" and arg1 == self.unit ) then
		ArenaLive_HealPredictionBar_Update(self);
	elseif ( event == "UNIT_ABSORB_AMOUNT_CHANGED" and arg1 == self.unit ) then
		ArenaLive_AbsorbBar_Update(self);
	elseif ( event == "UNIT_AURA" ) and ( arg1 == self.unit ) then
		ArenaLive_PortraitOverlay_QueryAuras(self, arg1);
	elseif (event == "UNIT_THREAT_SITUATION_UPDATE" and self.threatIndicator ) then
		ArenaLive_ThreatIndicator_Update(self, ...);
	elseif ( event == "PLAYER_REGEN_ENABLED" and self.needAttributeUpdate ) then
		-- Just a safetycheck for testing if CombatLockdown of frames really fades with this event.
		if ( not InCombatLockdown() ) then
			self.needAttributeUpdate = nil;
			self:SetAttribute("unit", self.unit);
			
			-- For if we want to show some frames even if the unitID isn't currently active
			if ( self.registerUnitWatch ) then
				RegisterUnitWatch(self);
			end
		end	
	elseif ( event == "UNIT_FACTION" and arg1 == self.unit ) then
		ArenaLive_UnitFrame_NameUpdate(self);
	end
	
end



function ArenaLive_UnitFrame_OnShow (self)

	local portraitType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["portraitType"];
	local isPlayer = UnitIsPlayer(self.unit);
	if ( ( portraitType == "3D" ) and ( self.unit ) ) then
		self.threeDPortrait:SetUnit(self.unit);
		self.threeDPortrait:SetCamera(0);
	elseif ( ( portraitType == "class" ) and ( self.unit ) and ( not isPlayer ) ) then
		-- Fix for NPCs to show the portrait camera correctly.
		self.threeDPortrait:SetCamera(0);
	end
end



function ArenaLive_UnitFrame_Update (self)

	if (not self.unit) then
		return;
	end

	ArenaLive_UnitFrame_NameUpdate(self);
	ArenaLive_UnitFrame_PortraitUpdate(self);
	ArenaLive_HealthBar_Update(self.healthBar, self.unit);
	ArenaLive_PowerBar_Update(self.powerBar, self.unit);
	ArenaLive_HealPredictionBar_Update(self);
	ArenaLive_AbsorbBar_Update(self);
	ArenaLive_ThreatIndicator_Update(self)

end



-- TO DO: NAMEHANDLER
function ArenaLive_UnitFrame_NameUpdate (self)

	if (not self.name) or (not self.unit) then
		return;
	end
	
	local colourMode = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["nameColourMode"];
	local isPlayer = UnitIsPlayer(self.unit);
	
	if ( colourMode == "byClass" and isPlayer ) then
	
		local _, class = UnitClass(self.unit);
		
		if ( class ) then
			self.name:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b);
		end
		
	elseif ( colourMode == "byReaction" or ( colourMode == "byClass" and not isPlayer ) ) then
	
		local red, green, blue = UnitSelectionColor(self.unit);

		-- If the unit is a NPC that was tapped by another person, we reflect that in the name colour by colouring it grey.
		if ( not UnitPlayerControlled(self.unit) and UnitIsTapped(self.unit) and not UnitIsTappedByPlayer(self.unit) and not UnitIsTappedByAllThreatList(self.unit) ) then
			self.name:SetTextColor(0.5, 0.5, 0.5);
		else
			-- Blue names are very hard to read, so we switch to plain white. math.ceil for blue because somhow the colour isn't exactly 1, but 0,999... 
			if ( red == 0 and green == 0 and math.ceil(blue) == 1 ) then
				self.name:SetTextColor(1, 1, 1);
			else
				self.name:SetTextColor(red, green, blue);
			end
			
		end

	else
		self.name:SetTextColor(1, 1, 1);
	end
	
	if ( self.overrideName ) then
		self.name:SetText(GetUnitName(self.overrideName));
	else
		self.name:SetText(GetUnitName(self.unit));
	end

end

function ArenaLive_UnitFrame_UpdateReadyCheck (self)
	
	if ( not self.unit or not self.readyCheck ) then
		return;
	end

	local readyCheckStatus = GetReadyCheckStatus(self.unit);
	if ( UnitName(self.unit) and UnitIsConnected(self.unit) and readyCheckStatus ) then
		if ( readyCheckStatus == "ready" ) then
			ReadyCheck_Confirm(self.readyCheck, 1);
		elseif ( readyCheckStatus == "notready" ) then
			ReadyCheck_Confirm(self.readyCheck, 0);
		else -- "waiting"
			ReadyCheck_Start(self.readyCheck);
		end
	else
		self.readyCheck:Hide();
	end
end

-- PORTRAITHANDLER
function ArenaLive_UnitFrame_PortraitUpdate (self)

	if (not self.portrait) or (not self.unit) then
		return;
	end
	
	local frameName = self:GetName();
	local portraitType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["portraitType"];
	
	if ( portraitType == "3D" ) then
		self.portrait:SetTexture(0, 0, 0, 1);
		self.portrait:SetTexCoord(0, 1, 0, 1);
		self.threeDPortrait:Show();
		self.threeDPortrait:SetUnit(self.unit);
		self.threeDPortrait:SetCamera(0);
		
	elseif ( portraitType == "class" ) then
		
		local _, class = UnitClass(self.unit);
		local unitType = string.match(self.unit, "^([a-z]+)[0-9]+$") or self.unit;
		local unitNumber = tonumber(string.match(self.unit, "^[a-z]+([0-9]+)$"));
		local isPlayer = UnitIsPlayer(self.unit);
		
		if ( class and isPlayer ) then
			self.threeDPortrait:Hide();
			self.portrait:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
			self.portrait:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
		elseif (not isPlayer ) then
			-- If the unit is a NPC we fall back to 3D-Portrait.
			self.portrait:SetTexture(0, 0, 0, 1);
			self.portrait:SetTexCoord(0, 1, 0, 1);
			self.threeDPortrait:Show();
			self.threeDPortrait:SetUnit(self.unit);
			self.threeDPortrait:SetCamera(0);
		elseif ( unitType == "arena" and unitNumber ) then
			--[[ For ArenaFrames we can get the class via GetArenaOpponentSpec() and GetSpecializationInfoByID(). 
			Therefore we need to have a fallback option when "ARENA_PREP_OPPONENT_SPECIALIZATIONS" fires.]]--
			local numOpps = GetNumArenaOpponentSpecs();
			
			if (numOpps and unitNumber and unitNumber <= numOpps ) then
				local specID = GetArenaOpponentSpec(unitNumber)
				local _, _, _, _, _, _, class = GetSpecializationInfoByID(specID)
				
				if ( class ) then
					self.threeDPortrait:Hide();
					self.portrait:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
					self.portrait:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
				else
					self.portrait:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
					self.portrait:SetTexCoord(0, 1, 0, 1);
				end
			else
				self.portrait:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				self.portrait:SetTexCoord(0, 1, 0, 1);
			end
			
			
		end
		
	end
end

-- Reverse Fill Handler
function ArenaLive_UnitFrame_SetBarReverseFill(frame, state)

	if ( frame.healthBar ) then
		frame.healthBar:SetReverseFill(state);
	end
	
	if ( frame.powerBar ) then
		frame.powerBar:SetReverseFill(state);
	end
	
end

-- Functions to enable/disable support for click cast addons like clique etc.
function ArenaLive_UnitFrame_EnableClickCast(frame)
	
	-- If our addon is loaded before the click cast addon we need to set up the table for them
	if ( not ClickCastFrames ) then
		ClickCastFrames = {};
	end
	
	ClickCastFrames[frame] = true;

end

function ArenaLive_UnitFrame_DisableClickCast(frame)

	if ( ClickCastFrames ) then
		ClickCastFrames[frame] = nil;
	end

end