local addonName, L = ...;
local maxPartyMembers = 5;

function ALUF_PartyFrames_OnLoad (self)

	self.frameType = "partyFrames";
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");

end

local test = 0;
function ALUF_PartyFrames_OnEvent (self, event, ...)
	local arg1 = ...;
	
	if ( event == "ADDON_LOADED" and arg1 == addonName) then
		ALUF_PartyFrames_UpdateOptions(self);
	elseif ( event == "GROUP_ROSTER_UPDATE" ) then
		
		if ( InCombatLockdown() ) then
			return;
		end
		
		local isEnabled = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["enabled"];
		
		if ( not isEnabled ) then
			if ( self:IsShown() ) then
				self:Hide();
			end
			return;
		end
		
		local showInRaids = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["showInRaid"];
		local showInArena = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["showInArena"];
		local _, instanceType = IsInInstance();
		
		if (not IsInGroup() or ( IsInRaid() and not showInRaids ) ) then
			if ( instanceType == "arena" and showInArena ) then
				self:Show();
			else
				self:Hide();
			end
			
		else
			self:Show();
		end
	end

end

function ALUF_PartyFrames_UpdateOptions (self)
	
	if ( InCombatLockdown() ) then
		local frameName = self:GetName();
		ArenaLive_SystemMessage(string.format(L["errorFrameUpdateCombatLockdown"], frameName), true);
		return;
	end
	
	local isEnabled = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["enabled"];
	local isLocked = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["isLocked"];
	local targetIsShown = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["targetFrame"]["isShown"];	
	local targetPosition = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["targetFrame"]["position"];
	local castBarIsShown = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["castBar"]["isShown"];
	local castBarPosition = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["castBar"]["position"];
	local castHistoryIsShown = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["castHistory"]["isShown"];
	local auraIsShown = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["auraFrame"]["isShown"];
	local auraPosition = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["auraFrame"]["position"];
	local petFrameIsShown = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["petFrame"]["isShown"];
	local petFramePosition = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["petFrame"]["position"];
	local scale = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["scale"];
	local showPlayer = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["showPlayer"];
	
	if (not IsInGroup() or not isEnabled ) then
		self:Hide();
	else
		self:Show();
	end
	
	self:SetScale(scale);
	ArenaLive_FrameMover_UpdateVisible(ALUF_PartyFramesMover);

	for i = 1, maxPartyMembers do
		local frame, unit, targetUnit, petUnit;
		
		if ( i < 5 )then
			frame = _G["ALUF_PartyFrame"..i];
			unit = "party"..i;
			targetUnit = "party"..i.."target";
			petUnit = "partypet"..i; 
		else
			frame = _G["ALUF_PartyPlayerFrame"];
			unit = "player";
			targetUnit = "target";
			petUnit = "pet"; 
		end
		
		if ( not isEnabled or ( i == 5 and not showPlayer ) )then
			ArenaLive_UnitFrame_ResetFrame(frame);
			ArenaLive_Icon_ResetFrame(frame.icon1);
			ArenaLive_Icon_ResetFrame(frame.icon2);
			ArenaLive_CastBar_ResetFrame(frame.castBar);
			ArenaLive_CastHistory_ResetFrame(frame.castHistory);
			ArenaLive_AuraFrame_ResetFrame(frame.auraFrame);
			ArenaLive_UnitFrame_ResetFrame(frame.petFrame);
			frame:Hide();
			return;
		else
			ArenaLive_UnitFrame_UpdateUnit(frame, unit);
			
			if ( isLocked ) then
				RegisterUnitWatch(frame);
			else
				UnregisterUnitWatch(frame);
				frame:Show();
			end
			
			if ( targetIsShown ) then
				ArenaLive_UnitFrame_UpdateUnit(frame.targetFrame, targetUnit);
				frame.targetFrame:ClearAllPoints();
				
				if ( targetPosition == "RIGHT" ) then
					frame.targetFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 0, 5);
				elseif ( targetPosition == "LEFT" ) then
					frame.targetFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", -3, 5);
				end
			else
				ArenaLive_UnitFrame_ResetFrame(frame.targetFrame);
				frame.targetFrame:Hide();			
			end
			
			if ( castBarIsShown ) then
				ArenaLive_CastBar_UpdateUnit(frame.castBar, unit);
				frame.castBar:ClearAllPoints();
				
				-- If castbar and target are on the same position, we need to anchor the castbar to the targetframe
				if ( castBarPosition == targetPosition and targetIsShown ) then
					if ( castBarPosition == "RIGHT" ) then
						local targetFrameName = frame.targetFrame:GetName();
						frame.castBar:SetPoint("TOPLEFT", frame.targetFrame, "TOPRIGHT", 32, 3);
					elseif ( castBarPosition == "LEFT" ) then
						frame.castBar:SetPoint("TOPRIGHT", frame.targetFrame, "TOPLEFT", -6, 3);
					end
				else
					if ( castBarPosition == "RIGHT" ) then
						frame.castBar:SetPoint("TOPLEFT", frame, "TOPRIGHT", 25, -16);
					elseif ( castBarPosition == "LEFT" ) then
						frame.castBar:SetPoint("TOPRIGHT", frame, "TOPLEFT", -3, -16);
					end
				end
			else
				ArenaLive_CastBar_ResetFrame(frame.castBar);
			end
			
			if ( castHistoryIsShown and castBarIsShown ) then
				ArenaLive_CastHistory_UpdateUnit(frame.castHistory, unit);
				frame.castHistory:ClearAllPoints();
				
				if ( castBarPosition == "RIGHT" ) then
					frame.castHistory.moveDirection = "right";
					frame.castHistory:SetPoint("TOPRIGHT", frame.castBar.border, "BOTTOMRIGHT", 2, 0);
				else
					frame.castHistory.moveDirection = "left";
					frame.castHistory:SetPoint("TOPLEFT", frame.castBar.border, "BOTTOMLEFT", -2, 0);
				end
				ArenaLive_CastHistory_ResetAllIcons(frame.castHistory);
			else
				ArenaLive_CastHistory_ResetFrame(frame.castHistory);
			end
			
			if ( auraIsShown ) then
				ArenaLive_AuraFrame_UpdateUnit(frame.auraFrame, unit);
				frame.auraFrame:ClearAllPoints();
				
				if ( auraPosition == "BELOW" ) then
					frame.auraFrame:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 28, 3);
					ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"][self.frameType]["auraFrame"]["growUpwards"] = false;
				else
					frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 28, 2);
					ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"][self.frameType]["auraFrame"]["growUpwards"] = true;
				end
			else
				ArenaLive_AuraFrame_ResetFrame(frame.auraFrame);
			end
			
			if ( petFrameIsShown ) then				
				ArenaLive_UnitFrame_UpdateUnit(frame.petFrame, petUnit);
				
				frame.petFrame:ClearAllPoints();
				if ( petFramePosition == "RIGHT" ) then
					frame.petFrame:SetPoint("BOTTOMLEFT", frame, "TOPRIGHT", -10, -18);
				else
					frame.petFrame:SetPoint("BOTTOMRIGHT", frame, "TOPLEFT", 10, -18);
				end
			else
				ArenaLive_UnitFrame_ResetFrame(frame.petFrame);
				frame.petFrame:Hide();
			end
			
			
			ArenaLive_UnitFrame_Update(frame);
			ArenaLive_Icon_Update(frame.icon1);
			ArenaLive_Icon_UpdateUnit(frame.icon1, unit);
			ArenaLive_Icon_Update(frame.icon2);
			ArenaLive_Icon_UpdateUnit(frame.icon2, unit);
		end
	end	
	
	ALUF_PartyFrames_AdjustPosition (self)
end

function ALUF_PartyFrames_AdjustPosition (self)

	local growUpwards = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["growUpwards"];
	local frameSpacing = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["frameSpacing"];
	local showPlayer = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["showPlayer"];
	
	local point, relativePoint;
	if ( growUpwards ) then
		frameSpacing = frameSpacing;
		point = "BOTTOMLEFT"
		relativePoint = "TOPLEFT"
	else
		frameSpacing = -frameSpacing;
		point = "TOPLEFT"
		relativePoint = "BOTTOMLEFT"
	end
	
	for i = 1, maxPartyMembers do
	
		local frame = _G["ALUF_PartyFrame"..i];
		
		if ( i == 1 ) then
			frame:ClearAllPoints();
			
			if ( showPlayer ) then
				frame:SetPoint(point, _G["ALUF_PartyPlayerFrame"], relativePoint, 0, frameSpacing);
			else
				frame:SetPoint(point, self, relativePoint, 10, frameSpacing);
			end
			
		elseif ( i == 5 ) then
			frame = _G["ALUF_PartyPlayerFrame"];
			frame:ClearAllPoints();
			frame:SetPoint(point, self, relativePoint, 10, frameSpacing);
		else
			frame:ClearAllPoints();
			frame:SetPoint(point, _G["ALUF_PartyFrame"..i-1], relativePoint, 0, frameSpacing);
		end
	end
end

function ALUF_PartyFrame_OnLoad(self)

	local prefix = self:GetName();
	local id = self:GetID();
	local unit = "party"..id;
	
	ArenaLive_SecureUnitButton_OnLoad(self, nil, nil, true);
	
	self.PVPIcon = _G[prefix.."PVPIcon"];
	self.levelText = _G[prefix.."LevelText"];
	self.leaderIcon = _G[prefix.."LeaderIcon"];
	self.guideIcon = _G[prefix.."GuideIcon"];
	self.masterLooterIcon = _G[prefix.."MasterLooterIcon"];
	self.roleIcon = _G[prefix.."RoleIcon"];
	self.speakerFrame = _G[prefix.."SpeakerFrame"];
	self.readyCheck = _G[prefix.."ReadyCheck"];
	self.targetFrame = _G[prefix.."TargetFrame"];
	self.petFrame = _G[prefix.."PetFrame"];

	ArenaLive_UnitFrame_Initialise(self, "ArenaLiveUnitFrames", "partyFrames", _G[prefix.."Background"], _G[prefix.."Border"], _G[prefix.."Name"], _G[prefix.."Portrait"], _G[prefix.."3DPortrait"], _G[prefix.."PortraitOverlay"], _G[prefix.."HealthBar"], _G[prefix.."HealthBarText"], _G[prefix.."PowerBar"], _G[prefix.."PowerBarText"], nil, nil, _G[prefix.."HealPredictionBar"], _G[prefix.."AbsorbBar"], _G[prefix.."AbsorbBarOverlay"], _G[prefix.."AbsorbFullHPIndicator"], _G[prefix.."TargetIndicator"]);
	ArenaLive_UnitFrame_UpdateUnit(self, unit);
	ArenaLive_Icon_Initialise(_G[prefix.."Icon1"], "ArenaLiveUnitFrames", "partyFrames", 1, _G[prefix.."Icon1Texture"], _G[prefix.."Icon1Cooldown"]);
	ArenaLive_Icon_UpdateUnit(_G[prefix.."Icon1"], unit);
	ArenaLive_Icon_Initialise(_G[prefix.."Icon2"], "ArenaLiveUnitFrames", "partyFrames", 2, _G[prefix.."Icon2Texture"], _G[prefix.."Icon2Cooldown"]);
	ArenaLive_Icon_UpdateUnit(_G[prefix.."Icon2"], unit);
	ArenaLive_CastBar_Initialise(_G[prefix.."CastBar"], "ArenaLiveUnitFrames", "partyFrames", _G[prefix.."CastBarBorder"], _G[prefix.."CastBarBorderShieldGlow"], _G[prefix.."CastBarIcon"], _G[prefix.."CastBarText"], _G[prefix.."CastBarAnimationGroup"], _G[prefix.."CastBarAnimationGroupFadeOutAnimation"], false, true);
	ArenaLive_CastBar_UpdateUnit(_G[prefix.."CastBar"], unit);
	ArenaLive_CastHistory_Initialise(_G[prefix.."CastHistory"], "ArenaLiveUnitFrames", "partyFrames", 4, "ArenaLive_CastHistoryIconTemplate", "right");
	ArenaLive_CastHistory_UpdateUnit(_G[prefix.."CastHistory"], unit);
	ArenaLive_AuraFrame_Initialise(_G[prefix.."Auras"], "ArenaLiveUnitFrames", "partyFrames", _G[prefix.."AurasBuffFrame"], _G[prefix.."AurasDebuffFrame"], "ArenaLive_BuffTemplate", "ArenaLive_DebuffTemplate");	
	ArenaLive_AuraFrame_UpdateUnit(_G[prefix.."Auras"], unit);
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("PARTY_LEADER_CHANGED");
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED");
	self:RegisterEvent("MUTELIST_UPDATE");
	self:RegisterEvent("IGNORELIST_UPDATE");
	self:RegisterEvent("UNIT_FACTION");
	self:RegisterEvent("VOICE_START");
	self:RegisterEvent("VOICE_STOP");
	self:RegisterEvent("VOICE_STATUS_UPDATE");
	self:RegisterEvent("READY_CHECK");
	self:RegisterEvent("READY_CHECK_CONFIRM");
	self:RegisterEvent("READY_CHECK_FINISHED");
	self:RegisterEvent("UNIT_ENTERED_VEHICLE");
	self:RegisterEvent("UNIT_EXITED_VEHICLE");
	self:RegisterEvent("PARTY_MEMBER_ENABLE");
	self:RegisterEvent("PARTY_MEMBER_DISABLE");
	self:RegisterEvent("UNIT_CONNECTION");
	self:RegisterEvent("UNIT_PHASE");
	self:RegisterEvent("UNIT_LEVEL");
	self:RegisterEvent("UNIT_OTHER_PARTY_CHANGED");
	
end

function ALUF_PartyFrame_Update (self)
	
	if ( not self.unit ) then
		return;
	end
	local id = self:GetID();
	local unit;
	if ( id == 5 ) then
		unit = "player";
	else
		unit = "party"..id;
	end
	
	if ( UnitExists(unit) ) then
		
		ArenaLive_UnitFrame_Update(self);
		ArenaLive_UnitFrame_Update(self.petFrame);
		ArenaLive_Icon_Update(self.icon1);
		ArenaLive_Icon_Update(self.icon2);
		
		self.levelText:SetText(UnitLevel(unit));
		local lootMethod, lootMaster = GetLootMethod();
		if ( id == lootMaster ) then
			self.masterLooterIcon:Show();
		else
			self.masterLooterIcon:Hide();
		end
	end
	
	ALUF_PartyFrame_UpdateLeader(self);
	ALUF_PartyFrame_UpdatePvPStatus(self);
	ALUF_PartyFrame_UpdateVoiceStatus(self);
	ArenaLive_UnitFrame_UpdateReadyCheck(self);
	ALUF_PartyFrame_UpdateMemberPhase(self);
end

function ALUF_PartyFrame_OnEvent(self, event, ...)

	if ( not self.unit ) then
		return;
	end
	
	ArenaLive_UnitFrame_OnEvent(self, event, ...);
	
	local arg1, arg2, arg3 = ...;
	local selfID = self:GetID();
	local unit, petUnit;
	if ( selfID < 5 ) then
		unit = "party"..selfID;
		petUnit = "partypet"..selfID;
	else
		unit = "player";
		petUnit = "pet";
	end

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		if ( UnitExists(unit) ) then
			ALUF_PartyFrame_Update(self);
			ArenaLive_UnitFrame_Update(self.petFrame);
		end
	elseif ( event == "GROUP_ROSTER_UPDATE" ) then
		ALUF_PartyFrame_Update(self);
		ALUF_PartyFrame_UpdateAssignedRoles(self);
		return;
	elseif ( event == "PARTY_LEADER_CHANGED" ) then
		ALUF_PartyFrame_UpdateLeader(self);
	elseif ( event == "PARTY_LOOT_METHOD_CHANGED" ) then
		local lootMethod;
		local lootMaster;
		lootMethod, lootMaster = GetLootMethod();
		if ( selfID == lootMaster ) then
			self.masterLooterIcon:Show();
		else
			self.masterLooterIcon:Hide();
		end
	elseif ( event == "MUTELIST_UPDATE" or event == "IGNORELIST_UPDATE" ) then
		ALUF_PartyFrame_UpdateVoiceStatus(self);
	elseif ( event == "UNIT_FACTION" ) then
		if ( arg1 == unit ) then
			ALUF_PartyFrame_UpdatePvPStatus(self);
		end
	elseif ( event =="UNIT_PET" ) then
		if ( arg1 == unit ) then
			ArenaLive_UnitFrame_Update(self.petFrame);
		elseif ( UnitHasVehicleUI(unit) and UnitIsConnected(unit)) then
			ArenaLive_UnitFrame_UpdateUnit(self, petUnit);
			ArenaLive_UnitFrame_Update(self);
		end
	elseif ( event == "READY_CHECK" or event == "READY_CHECK_CONFIRM" ) then
		ArenaLive_UnitFrame_UpdateReadyCheck(self);
	elseif ( event == "READY_CHECK_FINISHED" ) then
		if ( UnitExists(unit) ) then
			local finishTime = DEFAULT_READY_CHECK_STAY_TIME;

			ReadyCheck_Finish(self.readyCheck, finishTime);
		end
	elseif ( event == "VOICE_START") then
		if ( arg1 == unit ) then
			speaker.timer = nil;
			UIFrameFadeIn(speaker, 0.2, speaker:GetAlpha(), 1);
			VoiceChat_Animate(speaker, 1);
		end
	elseif ( event == "VOICE_STOP" ) then
		if ( arg1 == unit ) then
			speaker.timer = VOICECHAT_DELAY;
			VoiceChat_Animate(speaker, nil);
		end
	elseif ( event == "VOICE_STATUS_UPDATE" ) then
		ALUF_PartyFrame_UpdateVoiceStatus(self);
	elseif ( event == "UNIT_CONNECTION" and arg1 == unit ) then
		ALUF_PartyFrame_Update(self);
	elseif ( event == "UNIT_ENTERED_VEHICLE" ) then
		if ( arg1 == unit ) then
			if ( arg2 and UnitIsConnected(unit) ) then
				ArenaLive_UnitFrame_UpdateUnit(self, petUnit);
				ArenaLive_UnitFrame_Update(self);
				ArenaLive_UnitFrame_UpdateUnit(self.petFrame, unit);
				ArenaLive_UnitFrame_Update(self.petFrame);
			else
				ArenaLive_UnitFrame_UpdateUnit(self, unit);
				ArenaLive_UnitFrame_Update(self);
			end
		end
	elseif ( event == "UNIT_EXITED_VEHICLE" ) then
		if ( arg1 == unit ) then
			ArenaLive_UnitFrame_UpdateUnit(self, unit);
			ArenaLive_UnitFrame_Update(self);
			ArenaLive_UnitFrame_UpdateUnit(self.petFrame, petUnit);
			ArenaLive_UnitFrame_Update(self.petFrame);
		end
	elseif ( event == "UNIT_LEVEL" ) then
		if ( arg1 == unit ) then
			self.levelText:SetText(UnitLevel(unit));
		end
	elseif ( event == "UNIT_PHASE" or event == "PARTY_MEMBER_ENABLE" or event == "PARTY_MEMBER_DISABLE" ) then
		if ( event ~= "UNIT_PHASE" or arg1 == unit ) then
			ALUF_PartyFrame_UpdateMemberPhase(self);
		end
	elseif ( event == "UNIT_OTHER_PARTY_CHANGED" and arg1 == unit ) then
		ALUF_PartyFrame_UpdateMemberPhase(self);
	end

end

function ALUF_PartyFrame_UpdateLeader (self)
	local id = self:GetID();

	if( UnitIsGroupLeader(self.unit) ) then
		if ( HasLFGRestrictions() ) then
			self.guideIcon:Show();
			self.leaderIcon:Hide();
		else
			self.leaderIcon:Show();
			self.guideIcon:Hide();
		end
	else
		self.guideIcon:Hide();
		self.leaderIcon:Hide();
	end
end

function ALUF_PartyFrame_UpdatePvPStatus (self)
	local factionGroup = UnitFactionGroup(self.unit);
	if ( UnitIsPVPFreeForAll(self.unit) ) then
		self.PVPIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA");
		self.PVPIcon:Show();	
	elseif ( factionGroup and factionGroup ~= "Neutral" and UnitIsPVP(self.unit) ) then
		self.PVPIcon:SetTexture("Interface\\GroupFrame\\UI-Group-PVP-"..factionGroup);
		self.PVPIcon:Show();
	else
		self.PVPIcon:Hide();
	end
end

function ALUF_PartyFrame_UpdateAssignedRoles (self)

	local role = UnitGroupRolesAssigned(self.unit);
	
	if ( role == "TANK" or role == "HEALER" or role == "DAMAGER") then
		self.roleIcon:SetTexCoord(GetTexCoordsForRoleSmallCircle(role));
		self.roleIcon:Show();
	else
		self.roleIcon:Hide();
	end
end

function ALUF_PartyFrame_UpdateVoiceStatus (self)

	if ( not UnitExists(self.unit) ) then
		--No need to update if the frame doesn't have a unit.
		return;
	end
	
	local id = self:GetID();
	local mode;
	local inInstance, instanceType = IsInInstance();
	
	if ( (instanceType == "pvp") or (instanceType == "arena") ) then
		mode = "Battleground";
	elseif ( IsInRaid() ) then
		mode = "raid";
	else
		mode = "party";
	end
	local status = GetVoiceStatus(self.unit, mode);
	local statusIcon = self.speakerFrame
	local muted = GetMuteStatus(self.unit, mode);
	local mutedIcon;
	
	if ( id < 5 )then
		mutedIcon = _G["ALUF_PartyFrame"..id.."SpeakerFrameMuted"];
		_G["ALUF_PartyFrame"..id.."SpeakerFrameOn"]:SetVertexColor(0.7, 0.7, 0.7);
	else
		mutedIcon = _G["ALUF_PartyPlayerFrameSpeakerFrameMuted"];
		_G["ALUF_PartyPlayerFrameSpeakerFrameOn"]:SetVertexColor(0.7, 0.7, 0.7);
	end

	
	if ( status ) then
		statusIcon:Show();
	else
		statusIcon:Hide();
	end
	if ( muted ) then
		mutedIcon:Show();
	else
		mutedIcon:Hide();
	end
	
	-- Update the talking speaker thingie if they are talking or not.
	local state = UnitIsTalking(UnitName("party"..id));
	if ( state ) then
		VoiceChat_Animate(self.speakerFrame, 1);
		self.speakerFrame:Show();
	else
		VoiceChat_Animate(self.speakerFrame, nil);
		self.speakerFrame:Hide();
	end
end

function ALUF_PartyFrame_UpdateMemberPhase(self)

	if ( not self.unit ) then
		return;
	end
	
	local inPhase = UnitInPhase(self.unit);
	
	if ( UnitInOtherParty(self.unit) ) then
		self:SetAlpha(0.5);
	elseif ( not inPhase and UnitIsConnected(self.unit) ) then
		self:SetAlpha(0.5);
	else
		self:SetAlpha(1);
	end
end


-- Target of Party Frame functions
function ALUF_PartyTargetFrame_OnLoad(self)
	local prefix = self:GetName();
	local id = self:GetParent():GetID();
	local unit = "party"..id.."target";
	self:SetScale(0.8);
	self.targetFeedbackUnit = "party"..id;
	ArenaLive_SecureUnitButton_OnLoad(self, unit, nil, true, nil);
	
	ArenaLive_UnitFrame_Initialise(self, addonName, "targetOfPartyFrame", _G[prefix.."Background"], _G[prefix.."Border"], _G[prefix.."Name"], _G[prefix.."Portrait"], _G[prefix.."3DPortrait"], nil, _G[prefix.."HealthBar"], nil, _G[prefix.."PowerBar"], nil, nil, nil, nil);
	ArenaLive_UnitFrame_UpdateUnit(self, unit);
	
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UNIT_TARGET");
	
end

-- Party Pet Frame
function ALUF_PartyPetFrame_OnLoad(self)
	local id = self:GetParent():GetID();
	local unit = "partypet"..id;
	local prefix = self:GetName();
	ArenaLive_SecureUnitButton_OnLoad(self, unit, nil, true, nil);

	ArenaLive_UnitFrame_Initialise(self, addonName, "partyPetFrames", _G[prefix.."Background"], _G[prefix.."Border"],  _G[prefix.."Name"], _G[prefix.."Portrait"], _G[prefix.."3DPortrait"], nil, _G[prefix.."HealthBar"], _G[prefix.."HealthBarText"], nil, nil, nil, nil, _G[prefix.."HealPredictionBar"], _G[prefix.."AbsorbBar"], _G[prefix.."AbsorbBarOverlay"], _G[prefix.."AbsorbFullHPIndicator"], _G[prefix.."TargetIndicator"]);
	
	self:RegisterEvent("UNIT_NAME_UPDATE");
	ArenaLive_UnitFrame_UpdateUnit (self, unit);
	self:SetScript("OnShow", ALUF_PartyPetFrame_OnShow);

end

function ALUF_PartyPetFrame_OnEvent(self, event, ...)
	local arg1 = ...;
	ArenaLive_UnitFrame_OnEvent(self, event, ...);
	if ( event == "UNIT_NAME_UPDATE" ) and ( arg1 == self.unit ) then
		ArenaLive_UnitFrame_Update(self);
	end

end

function ALUF_PartyPetFrame_OnShow(self)

	ArenaLive_UnitFrame_OnShow(self);
	ArenaLive_UnitFrame_Update(self);
end