local addonName, L = ...;
local wasInitialsed = false;
local reverseFilledFrames = 
	{
		[1] = "ALUF_TargetFrame",
		[2] = "ALUF_FocusFrame",
		[3] = "ALUF_ArenaEnemyFrame1",
		[4] = "ALUF_ArenaEnemyFrame2",
		[5] = "ALUF_ArenaEnemyFrame3",
		[6] = "ALUF_ArenaEnemyFrame4",
		[7] = "ALUF_ArenaEnemyFrame5",
	}
local DropDownOptions = 
	{
		["Portrait"] = {
			[1] = {
				["text"] = L["3DPortrait"],
				["value"] = "3D",
			},
			[2] = {
				["text"] = L["classIcon"],
				["value"] = "class",
			},
		},
		["IconType"] = {
			[1] = {
				["text"] = L["class"],
				["value"] = "class",		
			},
			[2] = {
				["text"] = L["race"],
				["value"] = "race",	
			},
			[3] = {
				["text"] = L["pvpTrinket"],
				["value"] = "trinket",		
			},
			[4] = {
				["text"] = L["racial"],
				["value"] = "racial",			
			},
			[5] = {
				["text"] = L["specialisation"],
				["value"] = "specialisation",			
			},
			[6] = {
				["text"] = L["dispelOrInterrupt"],
				["value"] = "interruptOrDispel",			
			},
			[7] = {
				["text"] = L["reaction"],
				["value"] = "reaction",		
			},
		},
		["HealthBarColourMode"] = {
			[1] = {
				["text"] = L["none"],
				["value"] = "none",	
				
			},
			[2] = {
				["text"] = L["classColour"],
				["value"] = "byClass",	
				
			},
			[3] = {
				["text"] = L["reactionColour"],
				["value"] = "byReaction",	
				
			},
			[3] = {
				["text"] = L["smoothByHPAmount"],
				["value"] = "smooth",	
				
			},
		},
		["NameColourMode"] = {
			[1] = {
				["text"] = L["none"],
				["value"] = "none",	
				
			},
			[2] = {
				["text"] = L["classColour"],
				["value"] = "byClass",	
				
			},
			[3] = {
				["text"] = L["reactionColour"],
				["value"] = "byReaction",	
				
			},
		},
		["Position"] = {
			[1] = {
				["text"] = L["above"],
				["value"] = "ABOVE",	
				
			},
			[2] = {
				["text"] = L["below"],
				["value"] = "BELOW",	
			},
			[3] = {
				["text"] = L["left"],
				["value"] = "LEFT",	
				
			},
			[4] = {
				["text"] = L["right"],
				["value"] = "RIGHT",	
			},
		},
	}

function ALUF_Options_UpdateReverseFill()
		-- Update reverse Bar fill.
		local frame;
		local state = ARENALIVE_DATABASE[addonName]["options"]["reverseFillBars"];
		for key, value in ipairs(reverseFilledFrames) do
			frame = _G[value];
			ArenaLive_UnitFrame_SetBarReverseFill(frame, state)
			ArenaLive_CastBar_SetBarReverseFill(frame.castBar, state)
		end
end
	
function ALUF_Options_OnLoad (self)

	-- Create a Panel in Blizzard AddOn Panel
	local version = ARENALIVE_DATABASE["options"]["version"];
	self.name = "ArenaLiveUnitFrames";
	self.okay = ALUF_Options_Okay;
	self.cancel = ALUF_Options_Cancel;
	InterfaceOptions_AddCategory(self);
	
	local prefix = self:GetName();
	self.iCBlocker = _G[prefix.."InCombatLockDownBlocker"];
	_G[prefix.."InCombatLockDownBlockerText"]:SetText(L["inCombatLockdown"]);
	
	--self:RegisterEvent("ADDON_LOADED");
	
	-- Set Title:
	ALUF_OptionsTitle:SetText(L["ALUF:"]);
end

function ALUF_Options_Initialise(self)

	ALUF_Options_CheckButtonInitialise(ALUF_OptionsHealPrediction, "enableHealPrediction", nil, nil, "showHealPrediction");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsAbsorb, "enableAbsorbDisplay", nil, nil, "showAbsorbDisplay");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsOnlyShowCastableBuffs, "onlyShowCastableBuffs", nil, nil, "onlyShowCastableBuffs");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsOnlyShowDispellableDebuffs, "onlyShowDispellableDebuffs", nil, nil, "onlyShowDispellableDebuffs");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsShowAllEnemyDebuffs, "showAllDebuffs", nil, nil, "showAllEnemyDebuffs");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsHideBlizzCastBar, "hideBlizzCastBar", nil, nil, "hideBlizzCastBar");
	ALUF_Options_CheckButtonInitialise(ALUF_OptionsReverseFillBars, "reverseFillBars", nil, nil, "reverseFillBars");

end

function ALUF_Options_OnEvent (self, event, ...)
	arg1 = ...;
	
	if ( event == "ADDON_LOADED" and arg1 == addonName ) then
		-- General Options:
		ALUF_Options_Initialise(self)
	end

end

function ALUF_Options_OnShow(self)

	if ( not wasInitialsed ) then
		ALUF_Options_Initialise(self)
		wasInitialsed = true;
	end

	if ( InCombatLockdown() ) then
		self.iCBlocker:Show();
	else
		self.iCBlocker:Hide();
	end
end

function ALUF_Options_Okay (self)
	local frame;
	
	if ( not InCombatLockdown() and wasInitialsed ) then
		-- General Options:
		frame = _G["ALUF_OptionsHealPrediction"];
		ARENALIVE_DATABASE[addonName]["options"]["showHealPrediction"] = frame:GetChecked();
		frame = _G["ALUF_OptionsAbsorb"];
		ARENALIVE_DATABASE[addonName]["options"]["showAbsorbDisplay"] = frame:GetChecked();
		frame = _G["ALUF_OptionsOnlyShowCastableBuffs"];
		ARENALIVE_DATABASE[addonName]["options"]["onlyShowCastableBuffs"] = frame:GetChecked();
		frame = _G["ALUF_OptionsOnlyShowDispellableDebuffs"];
		ARENALIVE_DATABASE[addonName]["options"]["onlyShowDispellableDebuffs"] = frame:GetChecked();
		frame = _G["ALUF_OptionsShowAllEnemyDebuffs"];
		ARENALIVE_DATABASE[addonName]["options"]["showAllEnemyDebuffs"] = frame:GetChecked();
		frame = _G["ALUF_OptionsHideBlizzCastBar"];
		ARENALIVE_DATABASE[addonName]["options"]["hideBlizzCastBar"] = frame:GetChecked();
		frame = _G["ALUF_OptionsReverseFillBars"];
		ARENALIVE_DATABASE[addonName]["options"]["reverseFillBars"] = frame:GetChecked();
		
		-- Update Reverse fill
		ALUF_Options_UpdateReverseFill()
		-- Update Blizzard Castbar shown status.
		ArenaLive_ToggleBlizzFrame(CastingBarFrame, ARENALIVE_DATABASE[addonName]["options"]["hideBlizzCastBar"]);
		ArenaLive_Database_UpdateSavedVariables(addonName, ArenaLiveUnitFrames_SavedVariables);
	end
end



function ALUF_Options_Cancel ()
	if ( not InCombatLockdown() ) then
		-- General Options:
		ALUF_OptionsHealPrediction:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["showHealPrediction"]);
		ALUF_OptionsAbsorb:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["showAbsorbDisplay"]);
		ALUF_OptionsOnlyShowCastableBuffs:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["onlyShowCastableBuffs"]);
		ALUF_OptionsOnlyShowDispellableDebuffs:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["onlyShowDispellableDebuffs"]);
		ALUF_OptionsShowAllEnemyDebuffs:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["showAllEnemyDebuffs"]);
		ALUF_OptionsHideBlizzCastBar:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["hideBlizzCastBar"]);
		ALUF_OptionsReverseFillBars:SetChecked(ARENALIVE_DATABASE[addonName]["options"]["reverseFillBars"]);
	end
end



-- CHECKBUTTONS
function ALUF_Options_CheckButtonInitialise (self, localisationKey, frameType, nestedLevel, affectedVariable)
	
	local buttonName = self:GetName();
	local textString = _G[buttonName.."Text"];

	textString:SetText(L[localisationKey]);
	
	if ( not frameType ) then
		self:SetChecked(ARENALIVE_DATABASE[addonName]["options"][affectedVariable]);
	elseif ( frameType and not nestedLevel ) then
		self:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][affectedVariable]);
	elseif ( frameType and nestedLevel ) then
		self:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][nestedLevel][affectedVariable]);
	end
	
	
end



-- EDITBOXES
function ALUF_Options_EditBoxInitialise(self, localisationKey, frameType, nestedLevel, affectedVariable)

	local editBoxName = self:GetName();
	local textString = _G[editBoxName.."Text"];
	
	textString:SetText(L[localisationKey]);

	if ( not frameType ) then
		self:SetText(ARENALIVE_DATABASE[addonName]["options"][affectedVariable]);
	elseif ( frameType and not nestedLevel ) then
		self:SetText(ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][affectedVariable]);
	elseif ( frameType and nestedLevel ) then
		self:SetText(ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][nestedLevel][affectedVariable]);
	end
	
	-- We need to reset the EditBox's cursor position, so that the text is shown correctly.
	self:SetCursorPosition(0);
	
end

function ALUF_Options_EditBoxCheckMinMaxValues(self, minValue, maxValue)

	local value = tonumber(self:GetText());
	
	if ( not value ) then
		value = minValue;
		self:SetText(minValue);
	end
	
	if ( minValue and value < minValue ) then
		self:SetText(minValue);
	end
	
	if ( maxValue and value > maxValue ) then
		self:SetText(maxValue);
	end
	
	self:SetCursorPosition(0);
end

-- SLIDER
function  ALUF_Options_SliderInitialise(self, localisationKey, frameType, nestedLevel, affectedVariable)

	local prefix = self:GetName();
	local title = _G[prefix.."Title"];
	local currText = _G[prefix.."Curr"];
	local minText = _G[prefix.."Low"];
	local maxText = _G[prefix.."High"];
	local currValue;
	local minValue, maxValue = self:GetMinMaxValues();
	
	if ( not frameType ) then
		currValue = ARENALIVE_DATABASE[addonName]["options"][affectedVariable];
	elseif ( frameType and not nestedLevel ) then
		currValue = ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][affectedVariable];
	elseif ( frameType and nestedLevel ) then
		currValue = ARENALIVE_DATABASE[addonName]["frameTypes"][frameType][nestedLevel][affectedVariable];
	end
	
	
	
	if ( self.valueFactor ) then
		minValue = minValue * tonumber(self.valueFactor);
		maxValue = maxValue * tonumber(self.valueFactor);
		
		currValue = currValue / tonumber(self.valueFactor);
		self:SetValue(currValue);
		currValue = currValue * tonumber(self.valueFactor);
	else
		self:SetValue(currValue);
	end
	
	
	title:SetText(L[localisationKey]);
	minText:SetText(minValue);
	currText:SetText(currValue);
	maxText:SetText(maxValue);
end



function ALUF_Options_SliderOnValueChanged(self)
	
	local prefix = self:GetName();
	local currText = _G[prefix.."Curr"];
	local currValue = self:GetValue();
	
	if ( self.valueFactor ) then
		currValue = currValue * tonumber(self.valueFactor);
	end	
	
	currText:SetText(currValue);
end

function ALUF_Options_SliderGetValue(slider)
	
	if ( not slider ) then
		return;
	end
	
	local value = slider:GetValue();
	if ( slider.valueFactor ) then
		value = value * slider.valueFactor
	end

	return value;
end

function ALUF_Options_SliderSetValue(slider)
	
	if ( not slider ) then
		return;
	end
	
	local value = slider:GetValue();
	
	if ( slider.valueFactor ) then
		value = value / slider.valueFactor
	end

	return value;
end

-- DROPDOWNS
local info = {};
function ALUF_Options_DropDownInitialise(self)

	local title = _G[self:GetName().."Title"];
	local dropdownType = self.dropDownType;
	
	-- Set the Title of the DropDown:
	if ( self.dropDownType == "Portrait" ) then
		title:SetText(L["portraitType:"]);
	elseif ( self.dropDownType == "HealthBarColourMode" ) then
		title:SetText(L["healthBarColourMode:"]);
	elseif ( self.dropDownType == "IconType" ) then
		local id = self:GetID();
		title:SetText(L["Icon"..id..":"]);
	elseif ( self.dropDownType == "NameColourMode" ) then
		title:SetText(L["NameColourMode:"]);
	elseif ( self.dropDownType == "Position" ) then
		title:SetText(L["position:"]);
	elseif ( self.dropDownType == "PartyTargetPosition" ) then
		title:SetText(L["partyTargetPosition"]);
		dropdownType = "Position";
	elseif ( self.dropDownType == "PartyPetPosition" ) then
		title:SetText(L["partyPetPosition"]);
		dropdownType = "Position";
	end
	
	for key, value in ipairs(DropDownOptions[dropdownType]) do
		-- Some frames do not have all possible entries, so we filter it with the ingoreEntries table. The table is defined in the OnLoad Script for every DropDown in the options.xml
		if ( not self.ignoreEntries or ( self.ignoreEntries and not self.ignoreEntries[key] ) ) then	
			info = UIDropDownMenu_CreateInfo();
			info.text = value.text;
			info.value = value.value;
			info.func = ALUF_Options_DropDownSetValue;
			info.arg1 = self; -- dropDownFrame in ALUF_Options_DropDownSetValue
			UIDropDownMenu_AddButton(info);
		end
	end

end



function ALUF_Options_DropDownSetValue(self, dropDownFrame)
	UIDropDownMenu_SetSelectedValue(dropDownFrame, self.value, false);
end


local needreloadUI = false;
function ALUF_Options_SetUIReload()

	-- If UI was already flagged for reload, we don't need to open a Popup.
	if ( needreloadUI ) then
		return;
	else
		needreloadUI = true;
		ArenaLive_Popup_OpenPopup("ACTIONNEEDSUIRELOAD")
	end

end

function ALUF_Options_CheckNeedUIReload(frameType, newFrameStatus)

	local currFrameStatus = ARENALIVE_DATABASE[addonName]["frameTypes"][frameType]["enabled"]
	
	-- If the UI was already flagged for reload, we don't need to do anything here.
	if ( not needreloadUI ) then	
		--[[ We only need to force the UI reload, when we want to switch from an enabled to a disabled unit frame,
		because by hiding the blizzard frames and showing them again, we would taint them. ]]--
		if ( currFrameStatus and newFrameStatus ~= currFrameStatus ) then
			ALUF_Options_SetUIReload()
		end
	end
end