local addonName, L = ...;
local wasInitialsed = false;

function ALUF_OptionsTargetFrame_OnLoad(self)
	-- Create a sub panel in ArenaLiveUnitFrame addon panel
	self.name= L["targetFrame"];
	self.okay = ALUF_OptionsTargetFrame_Okay;
	self.cancel = ALUF_OptionsTargetFrame_Cancel;
	self.parent = "ArenaLiveUnitFrames";
	InterfaceOptions_AddCategory(self);	

	local prefix = self:GetName();
	self.iCBlocker = _G[prefix.."InCombatLockDownBlocker"];
	_G[prefix.."InCombatLockDownBlockerText"]:SetText(L["inCombatLockdown"]);	
	
	-- Set Localisation:
	ALUF_TargetFrame_OptionsTitle:SetText(L["ALUFTargetFrameTitle"]);
	ALUF_TargetFrame_OptionsCastBarTitle:SetText(L["castBar:"]);
	ALUF_TargetFrame_OptionsAuraTitle:SetText(L["auras:"]);
	
	--self:RegisterEvent("ADDON_LOADED");
end

function ALUF_OptionsTargetFrame_Initialise(self)
	local prefix = self:GetName();
	local checkbutton, editbox, slider, dropdown, selectedValue;
	
	-- Initialise Checkbuttons
	checkbutton = _G[prefix.."IsEnabled"];
	ALUF_Options_CheckButtonInitialise(checkbutton, "enableFrame", "targetFrame", nil, "enabled");
	
	checkbutton = _G[prefix.."IsLocked"];
	ALUF_Options_CheckButtonInitialise(checkbutton, "lockFrame", "targetFrame", nil, "isLocked");
	
	checkbutton = _G[prefix.."ShowCastBar"];
	ALUF_Options_CheckButtonInitialise(checkbutton, "showCastbar", "targetFrame", "castBar", "isShown");

	checkbutton = _G[prefix.."ShowCastHistory"];
	ALUF_Options_CheckButtonInitialise(checkbutton, "showCastHistory", "targetFrame", "castHistory", "isShown");	

	checkbutton = _G[prefix.."ShowAuras"];
	ALUF_Options_CheckButtonInitialise(checkbutton, "showAuras", "targetFrame", "auraFrame", "isShown");
	
	checkbutton = nil;
	
	
	-- Initialise Editboxes
	editbox = _G[prefix.."HealthBarText"];
	ALUF_Options_EditBoxInitialise(editbox, "shownTextinHealthBar:", "targetFrame", nil, "healthBarTextString");

	editbox = _G[prefix.."PowerBarText"];
	ALUF_Options_EditBoxInitialise(editbox, "shownTextinPowerBar:", "targetFrame", nil, "powerBarTextString");
	
	-- Initialise Sliders
	slider = _G[prefix.."FrameScale"];
	slider.valueFactor = 0.1;
	ALUF_Options_SliderInitialise(slider, "frameScale", "targetFrame", nil, "scale");

	slider = _G[prefix.."CastHistoryShownIcons"];
	ALUF_Options_SliderInitialise(slider, "maxCastHistoryIcons", "targetFrame", "castHistory", "shownIcons");	
	
	slider = _G[prefix.."CastHistoryIconSize"];
	ALUF_Options_SliderInitialise(slider, "castHistoryIconSize", "targetFrame", "castHistory", "iconSize");
	
	slider = _G[prefix.."NormalAuraIconSize"];
	ALUF_Options_SliderInitialise(slider, "normalAuraIconSize", "targetFrame", "auraFrame", "iconSize");

	slider = _G[prefix.."LargeAuraIconSize"];
	ALUF_Options_SliderInitialise(slider, "largeAuraIconSize", "targetFrame", "auraFrame", "largeIconSize");
	
	slider = nil;
	
	-- Initialise DropDown menus
	dropdown = _G[prefix.."PortraitType"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["portraitType"];
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);
	
	dropdown = _G[prefix.."Icon1"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon1Type"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);

	dropdown = _G[prefix.."Icon2"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon2Type"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);

	dropdown = _G[prefix.."HealthBarColour"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["healthBarColourMode"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);

	dropdown = _G[prefix.."NameColour"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["nameColourMode"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);
	
	dropdown = _G[prefix.."CastBarPosition"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castBar"]["position"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);
	
	dropdown = _G[prefix.."AuraPosition"];
	selectedValue = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["position"];	
	UIDropDownMenu_Initialize(dropdown, ALUF_Options_DropDownInitialise);
	UIDropDownMenu_SetSelectedValue(dropdown, selectedValue, false);
	
	dropdown = nil;
end

function ALUF_OptionsTargetFrame_OnEvent(self, event, ...)
	local arg1 = ...;
	
	if ( event == "ADDON_LOADED" and arg1 == addonName ) then
		ALUF_OptionsTargetFrame_Initialise(self)
	end

end

function ALUF_OptionsTargetFrame_OnShow(self)

	if ( not wasInitialsed ) then
		ALUF_OptionsTargetFrame_Initialise(self)
		wasInitialsed = true;
	end

	if ( InCombatLockdown() ) then
		self.iCBlocker:Show();
	else
		self.iCBlocker:Hide();
	end
end


function ALUF_OptionsTargetFrame_Okay(self)
	local prefix = self:GetName();
	local frame;
	
	if ( not InCombatLockdown() and wasInitialsed ) then	
		
		-- CHECKBUTTONS
		frame = _G[prefix.."IsEnabled"];
		--[[ We need to check, if we have to reload the UI here, because we taint the Blizz frame in the reactivation process.
		Once the UI is reloaded, the Blizz frame shouldn't be taitned anymore because then we don't try to hide him. ]]--
		ALUF_Options_CheckNeedUIReload("targetFrame", frame:GetChecked());
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["enabled"] = frame:GetChecked();
		
		frame = _G[prefix.."IsLocked"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["isLocked"] = frame:GetChecked();
		
		frame = _G[prefix.."ShowCastBar"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castBar"]["isShown"] = frame:GetChecked();
		
		frame = _G[prefix.."ShowCastHistory"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["isShown"] = frame:GetChecked();
		
		frame = _G[prefix.."ShowAuras"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["isShown"] = frame:GetChecked();
		
		
		-- EDITBOXES
		frame = _G[prefix.."HealthBarText"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["healthBarTextString"] = frame:GetText();
		
		frame = _G[prefix.."PowerBarText"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["powerBarTextString"] = frame:GetText();
		
		
		-- SLIDERS
		frame = _G[prefix.."FrameScale"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["scale"] = ALUF_Options_SliderGetValue(frame);
		
		frame = _G[prefix.."CastHistoryShownIcons"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["shownIcons"] = ALUF_Options_SliderGetValue(frame);

		frame = _G[prefix.."CastHistoryIconSize"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["iconSize"] = ALUF_Options_SliderGetValue(frame);

		frame = _G[prefix.."NormalAuraIconSize"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["iconSize"] = ALUF_Options_SliderGetValue(frame);

		frame = _G[prefix.."LargeAuraIconSize"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["largeIconSize"] = ALUF_Options_SliderGetValue(frame);
		
		
		-- DROPDOWN MENUS
		frame = _G[prefix.."PortraitType"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["portraitType"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."Icon1"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon1Type"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."Icon2"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon2Type"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."HealthBarColour"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["healthBarColourMode"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."NameColour"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["nameColourMode"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."CastBarPosition"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castBar"]["position"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = _G[prefix.."AuraPosition"];
		ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["position"] = UIDropDownMenu_GetSelectedValue(frame);
		
		frame = nil;
		
		-- Updapte the frame and saved variables
		ArenaLive_Database_UpdateSavedVariables(addonName, ArenaLiveUnitFrames_SavedVariables);
		
		local scale = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["scale"];
		local enabled = ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["enabled"];
		
		-- Hide Blizzard Frame if needed
		if ( enabled and not ArenaLive_BlizzFrameHider_CheckHiddenStatus("TargetFrame")) then
			ArenaLive_BlizzFrameHider_AddFrame("TargetFrame");
		end	
		
		ALUF_TargetFrame:SetScale(scale);
		ArenaLive_UnitFrame_Update(ALUF_TargetFrame);
		ArenaLive_Icon_Update(ALUF_TargetFrameIcon1);
		ArenaLive_Icon_Update(ALUF_TargetFrameIcon2);
		ALUF_TargetFrame_AdjustAttachments(ALUF_TargetFrame);
	end
end



function ALUF_OptionsTargetFrame_Cancel(self)
	
	local prefix = self:GetName();
	local frame;
	if ( not InCombatLockdown() ) then
		-- CHECKBUTTONS
		frame = _G[prefix.."IsEnabled"];
		frame:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["enabled"]);
		
		frame = _G[prefix.."IsLocked"];
		frame:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["isLocked"]);
		
		frame = _G[prefix.."ShowCastBar"];
		frame:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castBar"]["isShown"]);
		
		frame = _G[prefix.."ShowCastHistory"];
		frame:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["isShown"]);
		
		frame = _G[prefix.."ShowAuras"];
		frame:SetChecked(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["isShown"]);
		
		
		-- EDITBOXES
		frame = _G[prefix.."HealthBarText"];
		frame:SetText(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["healthBarTextString"]);
		
		frame = _G[prefix.."PowerBarText"];
		frame:SetText(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["powerBarTextString"]);
		
		
		-- SLIDERS
		frame = _G[prefix.."FrameScale"];
		ALUF_Options_SliderSetValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["scale"]);
			
		frame = _G[prefix.."CastHistoryShownIcons"];
		frame:SetValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["shownIcons"]);
			
		frame = _G[prefix.."CastHistoryIconSize"];
		frame:SetValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castHistory"]["iconSize"]);

		frame = _G[prefix.."NormalAuraIconSize"];
		frame:SetValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["iconSize"]);

		frame = _G[prefix.."LargeAuraIconSize"];
		frame:SetValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["largeIconSize"]);
		
		
		-- DROPDOWN MENUS
		frame = _G[prefix.."PortraitType"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["portraitType"]);
		
		frame = _G[prefix.."Icon1"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon1Type"]);
		
		frame = _G[prefix.."Icon2"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["icon2Type"]);
		
		frame = _G[prefix.."HealthBarColour"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["healthBarColourMode"]);
		
		frame = _G[prefix.."NameColour"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["nameColourMode"]);
		
		frame = _G[prefix.."CastBarPosition"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["castBar"]["position"]);
		
		frame = _G[prefix.."AuraPosition"];
		UIDropDownMenu_SetSelectedValue(ARENALIVE_DATABASE[addonName]["frameTypes"]["targetFrame"]["auraFrame"]["position"]);
		
		frame = nil;
	end
end